# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""beir dataset."""

from __future__ import annotations

import functools
import json
import os
from typing import Mapping

import numpy as np
import tensorflow_datasets.public_api as tfds

_BASE_DOWNLOAD_URL = (
    'https://public.ukp.informatik.tu-darmstadt.de/thakur/BEIR/datasets'
)


class BeirConfig(tfds.core.BuilderConfig):
  """BuilderConfig for BEIR datasets."""

  def __init__(
      self,
      name: str,
      download_url: str,
      qrel_splits: Mapping[str, str],
      subdir=None,
      **kwargs,
  ):
    """BuilderConfig for Beir.

    Args:
      name: Name for the config.
      download_url: Path to zip file of corpus, quries, and qrels.
      qrel_splits: Path to splits of TSV files that contain three columns, i.e.
        the query-id, corpus-id and score in this order.
      subdir: The sub-directory to use.
      **kwargs: keyword arguments forwarded to super.
    """
    super(BeirConfig, self).__init__(name=name, **kwargs)

    self.download_url = download_url
    self.qrel_splits = qrel_splits
    self.subdir = subdir


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for beir dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  BUILDER_CONFIGS = [
      BeirConfig(
          'msmarco',
          download_url=f'{_BASE_DOWNLOAD_URL}/msmarco.zip',
          qrel_splits={
              'train': 'train.tsv',
              'validation': 'dev.tsv',
              'test': 'test.tsv',
          },
      ),
      BeirConfig(
          'trec_covid',
          download_url=f'{_BASE_DOWNLOAD_URL}/trec-covid.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'nfcorpus',
          download_url=f'{_BASE_DOWNLOAD_URL}/nfcorpus.zip',
          qrel_splits={
              'train': 'train.tsv',
              'validation': 'dev.tsv',
              'test': 'test.tsv',
          },
      ),
      BeirConfig(
          'nq',
          download_url=f'{_BASE_DOWNLOAD_URL}/nq.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'hotpotqa',
          download_url=f'{_BASE_DOWNLOAD_URL}/hotpotqa.zip',
          qrel_splits={
              'train': 'train.tsv',
              'validation': 'dev.tsv',
              'test': 'test.tsv',
          },
      ),
      BeirConfig(
          'fiqa',
          download_url=f'{_BASE_DOWNLOAD_URL}/fiqa.zip',
          qrel_splits={
              'train': 'train.tsv',
              'validation': 'dev.tsv',
              'test': 'test.tsv',
          },
      ),
      BeirConfig(
          'arguana',
          download_url=f'{_BASE_DOWNLOAD_URL}/arguana.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'webis_touche2020',
          download_url=f'{_BASE_DOWNLOAD_URL}/webis-touche2020.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'cqadupstack.android',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='android',
      ),
      BeirConfig(
          'cqadupstack.english',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='english',
      ),
      BeirConfig(
          'cqadupstack.gaming',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='gaming',
      ),
      BeirConfig(
          'cqadupstack.gis',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='gis',
      ),
      BeirConfig(
          'cqadupstack.mathematica',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='mathematica',
      ),
      BeirConfig(
          'cqadupstack.physics',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='physics',
      ),
      BeirConfig(
          'cqadupstack.programmers',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='programmers',
      ),
      BeirConfig(
          'cqadupstack.stats',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='stats',
      ),
      BeirConfig(
          'cqadupstack.tex',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='tex',
      ),
      BeirConfig(
          'cqadupstack.unix',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='unix',
      ),
      BeirConfig(
          'cqadupstack.webmasters',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='webmasters',
      ),
      BeirConfig(
          'cqadupstack.wordpress',
          download_url=f'{_BASE_DOWNLOAD_URL}/cqadupstack.zip',
          qrel_splits={'test': 'test.tsv'},
          subdir='wordpress',
      ),
      BeirConfig(
          'quora',
          download_url=f'{_BASE_DOWNLOAD_URL}/quora.zip',
          qrel_splits={'validation': 'dev.tsv', 'test': 'test.tsv'},
      ),
      BeirConfig(
          'dbpedia_entity',
          download_url=f'{_BASE_DOWNLOAD_URL}/dbpedia-entity.zip',
          qrel_splits={'validation': 'dev.tsv', 'test': 'test.tsv'},
      ),
      BeirConfig(
          'scidocs',
          download_url=f'{_BASE_DOWNLOAD_URL}/scidocs.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'fever',
          download_url=f'{_BASE_DOWNLOAD_URL}/fever.zip',
          qrel_splits={
              'train': 'train.tsv',
              'validation': 'dev.tsv',
              'test': 'test.tsv',
          },
      ),
      BeirConfig(
          'climate_fever',
          download_url=f'{_BASE_DOWNLOAD_URL}/climate-fever.zip',
          qrel_splits={'test': 'test.tsv'},
      ),
      BeirConfig(
          'scifact',
          download_url=f'{_BASE_DOWNLOAD_URL}/scifact.zip',
          qrel_splits={'train': 'train.tsv', 'test': 'test.tsv'},
      ),
  ]

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    # TODO(beir): Specifies the tfds.core.DatasetInfo object
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'query_id': tfds.features.Text(),
            'query': tfds.features.Text(),
            'query_metadata': tfds.features.Text(),
            'passage_id': tfds.features.Text(),
            'passage': tfds.features.Text(),
            'passage_metadata': tfds.features.Text(),
            'score': np.float32,
        }),
        homepage='https://github.com/beir-cellar/beir',
    )

  def _split_generators(
      self, dl_manager: tfds.download.DownloadManager, pipeline
  ):
    """Returns SplitGenerators."""
    beam = tfds.core.lazy_imports.apache_beam
    # Downloads the data and defines the splits.
    builder_config = self.builder_config
    root = dl_manager.download_and_extract(builder_config.download_url)
    basename = os.path.basename(builder_config.download_url)
    unzip_dir_name = os.path.splitext(basename)[0]
    subdir = builder_config.subdir if builder_config.subdir else ''
    data_dir = os.path.join(root, unzip_dir_name, subdir)

    query_pipeline = (
        pipeline
        | 'ReadQueryData'
        >> beam.io.textio.ReadFromText(os.path.join(data_dir, 'queries.jsonl'))
        | 'ParseQueryJson' >> beam.Map(_parse_query_json)
    )

    passage_pipeline = (
        pipeline
        | 'ReadPassageData'
        >> beam.io.textio.ReadFromText(os.path.join(data_dir, 'corpus.jsonl'))
        | 'ParsePassageJson' >> beam.Map(_parse_passage_json)
    )

    splits = {
        'query': (
            query_pipeline
            | 'Query' >> beam.ptransform_fn(self._generate_query_examples)()
        ),
        'passage': (
            passage_pipeline
            | 'Passage' >> beam.ptransform_fn(self._generate_passage_examples)()
        ),
    }

    # Creates splits for (query, document) pairs.
    for qrel_split, tsv_file in builder_config.qrel_splits.items():
      splits[
          qrel_split
      ] = pipeline | qrel_split.capitalize() >> beam.ptransform_fn(
          self._generate_qrel_examples
      )(
          qrel_path=os.path.join(data_dir, 'qrels', tsv_file),
          query_pipeline=query_pipeline,
          passage_pipeline=passage_pipeline,
      )

    return splits

  def _generate_query_examples(self, pipeline):
    """Generates examples for query split."""
    beam = tfds.core.lazy_imports.apache_beam
    return (
        pipeline
        | 'AddEmptyFeatures'
        >> beam.Map(
            functools.partial(
                _append_constant_features,
                mapping={
                    'passage_id': '',
                    'passage': '',
                    'passage_metadata': '{}',
                    'score': -1,
                },
            )
        )
        | 'GetHashKey' >> beam.Map(lambda x: (_get_hash(x), x))
    )

  def _generate_passage_examples(self, pipeline):
    """Generates examples for passage split."""
    beam = tfds.core.lazy_imports.apache_beam
    return (
        pipeline
        | 'AddEmptyFeatures'
        >> beam.Map(
            functools.partial(
                _append_constant_features,
                mapping={
                    'query_id': '',
                    'query': '',
                    'query_metadata': '{}',
                    'score': -1,
                },
            )
        )
        | 'GetHashKey' >> beam.Map(lambda x: (_get_hash(x), x))
    )

  def _generate_qrel_examples(
      self, pipeline, qrel_path, query_pipeline, passage_pipeline
  ):
    """Generates (query, passage) pair examples."""
    beam = tfds.core.lazy_imports.apache_beam
    qid_query = query_pipeline | 'QidAsKey' >> beam.Map(
        lambda x: (x['query_id'], x)  # pylint: disable=g-long-lambda
    )
    docid_passage = passage_pipeline | 'DocidAsKey' >> beam.Map(
        lambda x: (x['passage_id'], x)
    )

    # PCollection[qid, Tuple(docid, score)]
    qid_docid = (
        pipeline
        | beam.io.ReadFromText(qrel_path)
        | 'Split' >> beam.Map(lambda x: x.strip().split('\t')[:3])
        | 'FilterFirstLine' >> beam.Filter(lambda x: x[0] != 'query-id')
        | 'QidAsKeyAndDocid' >> beam.Map(lambda x: (x[0], (x[1], float(x[2]))))
    )

    def to_docid_query_fn(ex):
      rst = []
      for docid, score in ex['docids']:
        rst.append((
            docid,
            {  # pylint: disable=g-complex-comprehension
                **ex['query'][0],
                'score': score,
            },
        ))
      return rst

    # PCollection[docid, Dict].
    # Dict.keys() = ['query_id', 'query', 'query_metadata', 'score']
    docid_query = (
        ({
            'docids': qid_docid,
            'query': qid_query,
        })
        | 'GroupByQid' >> beam.CoGroupByKey()
        | 'DropQid' >> beam.Values()
        | 'FilterEmptyQid' >> beam.Filter(lambda x: x['query'] and x['docids'])
        | 'SetKeyDocid' >> beam.FlatMap(to_docid_query_fn)
    )

    return (
        ({
            'query': docid_query,
            'passage': docid_passage,
        })
        | 'GroupByDocid' >> beam.CoGroupByKey()
        | 'DropDocid' >> beam.Values()
        | 'FilterEmptyDocid'
        >> beam.Filter(lambda x: x['query'] and x['passage'])
        | 'MergeQueryAndPassage'
        >> beam.Map(
            lambda x: {  # pylint: disable=g-long-lambda
                **x['query'][0],
                **x['passage'][0],
            }
        )
        | 'GetHashKey' >> beam.Map(lambda x: (_get_hash(x), x))
    )

  def _generate_examples(self, pipeline):
    """Yields examples."""
    # Not used.
    pass


def _parse_query_json(text):
  """Parses query json object."""
  data = json.loads(text)
  return {
      'query_id': data['_id'],
      'query': data['text'],
      'query_metadata': json.dumps(data['metadata']),
  }


def _parse_passage_json(text):
  """Parses passage json object."""
  data = json.loads(text)
  metadata = data['metadata']
  if 'title' in data:
    metadata['title'] = data['title']
  return {
      'passage_id': data['_id'],
      'passage': data['text'],
      'passage_metadata': json.dumps(metadata),
  }


def _append_constant_features(features, mapping):
  """Appends constant features."""
  output = dict(features)
  output.update(mapping)
  return output


def _get_hash(features):
  """Returns a hash key for features."""
  s = '|'.join([
      str(features['query_id']),
      features['query'] or '',
      str(features['passage_id']),
      features['passage'] or '',
  ])
  return hash(s.encode())



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""DAS beamformed phantom images and paired clinical post-processed images."""

import csv
import datetime
import os
from etils import epath
import numpy as np
from tensorflow_datasets.core.utils import bool_utils
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_DATA_URL = {
    'phantom_data': 'https://research.repository.duke.edu/downloads/vt150j912',
    'mark_data': 'https://research.repository.duke.edu/downloads/4x51hj56d',
}

_DEFAULT_SPLITS = {
    'train': 'https://research.repository.duke.edu/downloads/tt44pn391',
    'test': 'https://research.repository.duke.edu/downloads/zg64tm441',
    'validation': 'https://research.repository.duke.edu/downloads/dj52w535x',
    'MARK': 'https://research.repository.duke.edu/downloads/wd375w77v',
    'A': 'https://research.repository.duke.edu/downloads/nc580n18d',
    'B': 'https://research.repository.duke.edu/downloads/7h149q56p',
}


class Builder(tfds.core.GeneratorBasedBuilder):
  """DAS beamformed phantom images and paired post-processed images."""

  VERSION = tfds.core.Version('2.0.0')
  RELEASE_NOTES = {
      '2.0.0': r'Fix timestamp_id from %Y%m%d%H%M%S to posix timestamp.',
      '1.0.1': 'Fixes parsing of boolean field `harmonic`.',
      '1.0.0': 'Initial release.',
  }

  def __init__(self, custom_csv_splits=None, **kwargs):
    """custom_csv_splits is a dictionary of { 'name': 'csvpaths'}."""
    super(Builder, self).__init__(**kwargs)
    self.custom_csv_splits = custom_csv_splits if custom_csv_splits else {}

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'das': {
                'dB': tfds.features.Tensor(shape=(None,), dtype=np.float32),
                'real': tfds.features.Tensor(shape=(None,), dtype=np.float32),
                'imag': tfds.features.Tensor(shape=(None,), dtype=np.float32),
            },
            'dtce': tfds.features.Tensor(shape=(None,), dtype=np.float32),
            'f0_hz': tfds.features.Tensor(shape=(), dtype=np.float32),
            'voltage': tfds.features.Tensor(shape=(), dtype=np.float32),
            'focus_cm': tfds.features.Tensor(shape=(), dtype=np.float32),
            'height': tfds.features.Tensor(shape=(), dtype=np.uint32),
            'width': tfds.features.Tensor(shape=(), dtype=np.uint32),
            'initial_radius': tfds.features.Tensor(shape=(), dtype=np.float32),
            'final_radius': tfds.features.Tensor(shape=(), dtype=np.float32),
            'initial_angle': tfds.features.Tensor(shape=(), dtype=np.float32),
            'final_angle': tfds.features.Tensor(shape=(), dtype=np.float32),
            'probe': tfds.features.Tensor(shape=(), dtype=np.str_),
            'scanner': tfds.features.Tensor(shape=(), dtype=np.str_),
            'target': tfds.features.Tensor(shape=(), dtype=np.str_),
            'timestamp_id': tfds.features.Tensor(shape=(), dtype=np.uint32),
            'harmonic': tfds.features.Tensor(shape=(), dtype=np.bool_),
        }),
        supervised_keys=('das/dB', 'dtce'),
        homepage='https://github.com/ouwen/mimicknet',
    )

  def _split_generators(self, dl_manager):
    downloads = _DEFAULT_SPLITS.copy()
    downloads.update(_DATA_URL)
    dl_paths = dl_manager.download_and_extract(downloads)
    splits = [
        tfds.core.SplitGenerator(  # pylint:disable=g-complex-comprehension
            name=name,
            gen_kwargs={
                'datapath': {
                    'mark_data': dl_paths['mark_data'],
                    'phantom_data': dl_paths['phantom_data'],
                },
                'csvpath': dl_paths[name],
            },
        )
        for name, _ in _DEFAULT_SPLITS.items()
    ]

    for name, csv_path in self.custom_csv_splits.items():
      splits.append(
          tfds.core.SplitGenerator(
              name=name,
              gen_kwargs={'datapath': dl_paths['data'], 'csvpath': csv_path},
          )
      )

    return splits

  def _generate_examples(self, datapath, csvpath):
    with epath.Path(csvpath).open() as f:
      reader = csv.DictReader(f)
      for row in reader:
        data_key = 'mark_data' if row['target'] == 'mark' else 'phantom_data'

        filepath = os.path.join(datapath[data_key], row['filename'])
        matfile = tfds.core.lazy_imports.scipy.io.loadmat(
            tf.io.gfile.GFile(filepath, 'rb')
        )

        iq = np.abs(np.reshape(matfile['iq'], -1))
        iq = iq / iq.max()
        iq = 20 * np.log10(iq)

        timestamp_id = datetime.datetime.strptime(
            row['timestamp_id'], '%Y%m%d%H%M%S'
        )
        timestamp = int(timestamp_id.timestamp())

        yield row['filename'], {
            'das': {
                'dB': iq.astype(np.float32),
                'real': np.reshape(matfile['iq'], -1).real.astype(np.float32),
                'imag': np.reshape(matfile['iq'], -1).imag.astype(np.float32),
            },
            'dtce': np.reshape(matfile['dtce'], -1).astype(np.float32),
            'f0_hz': row['f0'],
            'voltage': row['v'],
            'focus_cm': row['focus_cm'],
            'height': row['axial_samples'],
            'width': row['lateral_samples'],
            'initial_radius': row['initial_radius'],
            'final_radius': row['final_radius'],
            'initial_angle': row['initial_angle'],
            'final_angle': row['final_angle'],
            'probe': row['probe'],
            'scanner': row['scanner'],
            'target': row['target'],
            'timestamp_id': timestamp,
            'harmonic': bool_utils.parse_bool(row['harm']),
        }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for np_utils."""

import numpy as np
from tensorflow_datasets.core.utils import np_utils


def test_is_notebook():
  assert np_utils.to_np_shape(()) == ()  # pylint: disable=g-explicit-bool-comparison
  assert np_utils.to_np_shape((1, 2, 3)) == (1, 2, 3)
  assert np_utils.to_np_shape((None, 2, 3)) == (-1, 2, 3)
  assert np_utils.to_np_shape((None, None, None)) == (-1, -1, -1)


def test_np_map_fn():
  # Scalar to scalar.
  add_ten = lambda i: i + 10
  np.testing.assert_array_equal(
      np_utils.np_map_fn(add_ten, np.array([0, 1, 2])), np.array([10, 11, 12])
  )

  # Dicts to scalar.
  operation_on_dict = lambda d: d['a'] + d['b'] if d['a'] % 2 == 0 else 0
  result = np_utils.np_map_fn(
      operation_on_dict,
      {
          'a': np.array([0, 1, 2]),
          'b': np.array([5, 6, 7]),
      },
  )
  expected = np.array([5, 0, 9])
  np.testing.assert_array_equal(result, expected)

  # Dicts to dicts.
  operation_on_dict = lambda _: {'c': 1, 'd': 2}
  result = np_utils.np_map_fn(
      operation_on_dict,
      {
          'a': np.array([0, 1, 2]),
          'b': np.array([5, 6, 7]),
      },
  )
  expected = {
      'c': np.array([1, 1, 1]),
      'd': np.array([2, 2, 2]),
  }
  assert len(result) == 2
  np.testing.assert_array_equal(result['c'], expected['c'])
  np.testing.assert_array_equal(result['d'], expected['d'])



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""universal_dependencies dataset."""

from tensorflow_datasets.core.dataset_builders.conll import conllu_dataset_builder_utils as conllu_lib
from tensorflow_datasets.datasets.universal_dependencies import universal_dependencies_utils as ud_utils
import tensorflow_datasets.public_api as tfds

_DATA_URL = "https://raw.githubusercontent.com/UniversalDependencies/"


class Builder(tfds.dataset_builders.ConllUDatasetBuilder):
  """DatasetBuilder for universal_dependencies dataset."""

  BUILDER_CONFIGS = []
  for language in ud_utils.LANGS:
    BUILDER_CONFIGS.append(
        conllu_lib.get_universal_morphology_config(
            language=language,
            description=ud_utils.DESCRIPTIONS[language],
            features=conllu_lib.UNIVERSAL_DEPENDENCIES_FEATURES,
        )
    )

  VERSION = tfds.core.Version("1.0.1")
  RELEASE_NOTES = {
      "1.0.1": "Updated config names.",
      "1.0.0": (
          "Initial release, which corresponds to Universal Dependencies 2.10."
      ),
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.create_dataset_info(
        homepage="https://universaldependencies.org/",
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    paths = {}
    for split, split_paths in ud_utils.UD_FILEPATHS[
        self.builder_config.language
    ].items():
      paths[split] = ud_utils.prepare_ud_filepaths(
          path_prefix=_DATA_URL, filepaths=split_paths
      )

    paths_per_split = dl_manager.download_and_extract(paths)

    return {
        split: self._generate_examples(filepaths=split_paths)
        for split, split_paths in paths_per_split.items()
    }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Remote Sensing Image Scene Classification (RESISC45) Dataset."""

import os

from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_LABELS = [
    'airplane',
    'airport',
    'baseball_diamond',
    'basketball_court',
    'beach',
    'bridge',
    'chaparral',
    'church',
    'circular_farmland',
    'cloud',
    'commercial_area',
    'dense_residential',
    'desert',
    'forest',
    'freeway',
    'golf_course',
    'ground_track_field',
    'harbor',
    'industrial_area',
    'intersection',
    'island',
    'lake',
    'meadow',
    'medium_residential',
    'mobile_home_park',
    'mountain',
    'overpass',
    'palace',
    'parking_lot',
    'railway',
    'railway_station',
    'rectangular_farmland',
    'river',
    'roundabout',
    'runway',
    'sea_ice',
    'ship',
    'snowberg',
    'sparse_residential',
    'stadium',
    'storage_tank',
    'tennis_court',
    'terrace',
    'thermal_power_station',
    'wetland',
]

_URL = 'http://www.escience.cn/people/JunweiHan/NWPU-RESISC45.html'


class Builder(tfds.core.GeneratorBasedBuilder):
  """NWPU Remote Sensing Image Scene Classification (RESISC) Dataset."""

  VERSION = tfds.core.Version('3.0.0')

  MANUAL_DOWNLOAD_INSTRUCTIONS = """\
  Dataset can be downloaded from OneDrive:
  https://1drv.ms/u/s!AmgKYzARBl5ca3HNaHIlzp_IXjs
  After downloading the rar file, please extract it to the manual_dir.
  """

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(shape=[256, 256, 3]),
            'label': tfds.features.ClassLabel(names=_LABELS),
            'filename': tfds.features.Text(),
        }),
        supervised_keys=('image', 'label'),
        homepage=_URL,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    path = os.path.join(dl_manager.manual_dir, 'NWPU-RESISC45')
    if not tf.io.gfile.exists(path):
      raise AssertionError(
          'You must download the dataset manually from {}, '
          'extract it, and place it in {}.'.format(_URL, dl_manager.manual_dir)
      )
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={'path': path},
        ),
    ]

  def _generate_examples(self, path):
    """Yields examples."""
    for label in tf.io.gfile.listdir(path):
      for filename in tf.io.gfile.glob(os.path.join(path, label, '*.jpg')):
        example = {
            'image': filename,
            'label': label,
            'filename': os.path.basename(filename),
        }
        yield f'{label}_{os.path.basename(filename)}', example



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Text utilities.

`tfds` includes a set of `TextEncoder`s as well as a `Tokenizer` to enable
expressive, performant, and reproducible natural language research.
"""

from tensorflow_datasets.core.deprecated.text.subword_text_encoder import SubwordTextEncoder
from tensorflow_datasets.core.deprecated.text.text_encoder import ByteTextEncoder
from tensorflow_datasets.core.deprecated.text.text_encoder import TextEncoder
from tensorflow_datasets.core.deprecated.text.text_encoder import TextEncoderConfig
from tensorflow_datasets.core.deprecated.text.text_encoder import Tokenizer
from tensorflow_datasets.core.deprecated.text.text_encoder import TokenTextEncoder

__all__ = [
    "ByteTextEncoder",
    "SubwordTextEncoder",
    "TextEncoder",
    "TextEncoderConfig",
    "Tokenizer",
    "TokenTextEncoder",
]



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Imagenet validation with multi-label annotations (http://proceedings.mlr.press/v119/shankar20c.html).
"""

import json
import os

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
from tensorflow_datasets.datasets.imagenet2012 import imagenet_common
import tensorflow_datasets.public_api as tfds

_MULTI_LABELS_URL = 'https://storage.googleapis.com/brain-car-datasets/imagenet-mistakes/human_accuracy_v3.0.0.json'


def _get_multi_labels_and_problematic_images(
    dl_manager: tfds.download.DownloadManager,
):
  """Returns multi-labels and problematic images from download json.

  Args:
    dl_manager: tfds.download.DownloadManager for downloading the json file

  Returns:
    val_annotated_images: Dictionary mapping image name to an inner dictionary
      containing the multi_label annotations for that image. The inner multi-
      label annotation dictionary has keys 'correct', 'wrong', or 'unclear'
      (keys will be missing if the image does not have a set of labels of the
      given type) and values that are lists of wnids.
    problematic_images: List of image names for problematic images.
    imagenet_m_2022:  List of image names comprising ImageNet-M 2022 evaluation
      slice.
  """
  with tf.io.gfile.GFile(dl_manager.download(_MULTI_LABELS_URL), 'r') as f:
    human_accuracy_data = json.load(f)
  val_annotated_images = {}
  prefix = 'ILSVRC2012_val_'
  len_prefix = len(prefix)
  for image_name in human_accuracy_data['initial_annots'].keys():
    if image_name[:len_prefix] == prefix:
      val_annotated_images[image_name] = human_accuracy_data['initial_annots'][
          image_name
      ]

  problematic_images = list(human_accuracy_data['problematic_images'].keys())
  imagenet_m_2022 = human_accuracy_data['imagenet_m']
  return val_annotated_images, problematic_images, imagenet_m_2022


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for imagenet2012_multilabel dataset."""

  VERSION = tfds.core.Version('3.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
      '2.0.0': 'Fixed ILSVRC2012_img_val.tar file.',
      '3.0.0': 'Corrected labels and ImageNet-M split.',
  }

  MANUAL_DOWNLOAD_INSTRUCTIONS = """\
  manual_dir should contain `ILSVRC2012_img_val.tar` file.
  You need to register on http://www.image-net.org/download-images in order
  to get the link to download the dataset.
  """

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    names_file = imagenet_common.label_names_file()
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(encoding_format='jpeg'),
            'original_label': tfds.features.ClassLabel(names_file=names_file),
            'correct_multi_labels': tfds.features.Sequence(
                tfds.features.ClassLabel(names_file=names_file)
            ),
            'wrong_multi_labels': tfds.features.Sequence(
                tfds.features.ClassLabel(names_file=names_file)
            ),
            'unclear_multi_labels': tfds.features.Sequence(
                tfds.features.ClassLabel(names_file=names_file)
            ),
            'is_problematic': tfds.features.Tensor(shape=(), dtype=np.bool_),
            'file_name': tfds.features.Text(),
        }),
        supervised_keys=('image', 'correct_multi_labels'),
        homepage='https://github.com/modestyachts/evaluating_machine_accuracy_on_imagenet',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    val_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_val.tar')
    if not tf.io.gfile.exists(val_path):
      raise AssertionError(
          'ImageNet requires manual download of the data. Please download '
          'the train and val set and place them into: {}'.format(val_path)
      )

    original_labels = imagenet_common.get_validation_labels(val_path)

    (multi_labels, problematic_images, imagenet_m_2022_errors) = (
        _get_multi_labels_and_problematic_images(dl_manager)
    )

    imagenet_m_2022 = dict(
        [(k, multi_labels[k]) for k in imagenet_m_2022_errors]
    )

    return {
        'validation': self._generate_examples(
            archive=dl_manager.iter_archive(val_path),
            original_labels=original_labels,
            multi_labels=multi_labels,
            problematic_images=problematic_images,
        ),
        'imagenet_m': self._generate_examples(
            archive=dl_manager.iter_archive(val_path),
            original_labels=original_labels,
            multi_labels=imagenet_m_2022,
            problematic_images=problematic_images,
        ),
    }

  def _generate_examples(
      self, archive, original_labels, multi_labels, problematic_images
  ):
    """Yields (key, example) tuples from the dataset."""
    for fname, fobj in archive:
      if fname not in multi_labels:
        # Image is not in the annotated set of 20,000 images
        continue
      else:
        is_problematic = fname in problematic_images
        correct_multi_labels = []
        wrong_multi_labels = []
        unclear_multi_labels = []
        if 'correct' in multi_labels[fname]:
          correct_multi_labels = multi_labels[fname]['correct']
        if 'wrong' in multi_labels[fname]:
          wrong_multi_labels = multi_labels[fname]['wrong']
        if 'unclear' in multi_labels[fname]:
          unclear_multi_labels = multi_labels[fname]['unclear']

      record = {
          'file_name': fname,
          'image': fobj,
          'original_label': original_labels[fname],
          'correct_multi_labels': correct_multi_labels,
          'wrong_multi_labels': wrong_multi_labels,
          'unclear_multi_labels': unclear_multi_labels,
          'is_problematic': is_problematic,
      }
      yield fname, record



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""asqa dataset."""
import json

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds


def _features():
  return tfds.features.FeaturesDict({
      'sample_id': np.int64,
      'ambiguous_question': tfds.features.Text(
          doc='Disambiguated question from AmbigQA.'
      ),
      'qa_pairs': tfds.features.Sequence(
          tfds.features.FeaturesDict({
              'question': tfds.features.Text(),
              'short_answers': tfds.features.Sequence(
                  tfds.features.Text(),
                  doc='List of short answers from AmbigQA.',
              ),
              'context': tfds.features.Text(doc='Additional context provided.'),
              'wikipage': tfds.features.Text(
                  doc=(
                      'Title of the Wikipedia page the additional context was'
                      ' taken from.'
                  )
              ),
          }),
          doc='Q&A pairs from AmbigQA which are used for disambiguation.',
      ),
      'wikipages': tfds.features.Sequence(
          tfds.features.FeaturesDict({
              'title': tfds.features.Text(doc='Title of the Wikipedia page.'),
              'url': tfds.features.Text(doc='Link to the Wikipedia page.'),
          }),
          doc='List of Wikipedia pages visited by AmbigQA annotators.',
      ),
      'annotations': tfds.features.Sequence(
          tfds.features.FeaturesDict({
              'long_answer': tfds.features.Text(doc='Annotation.'),
              'knowledge': tfds.features.Sequence(
                  tfds.features.FeaturesDict({
                      'content': tfds.features.Text(
                          doc='A passage from Wikipedia.'
                      ),
                      'wikipage': tfds.features.Text(
                          doc=(
                              'Title of the Wikipedia page the passage was'
                              ' taken from.'
                          )
                      ),
                  }),
                  doc='List of additional knowledge pieces.',
              ),
          }),
          doc=(
              'Long-form answers to the ambiguous question constructed by ASQA'
              ' annotators.'
          ),
      ),
  })


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for asqa dataset."""

  VERSION = tfds.core.Version('2.0.0')
  RELEASE_NOTES = {
      '2.0.0': 'Sample ID goes from int32 (overflowing) to int64.',
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=_features(),
        supervised_keys=None,
        homepage='https://github.com/google-research/language/tree/master/language/asqa',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    file_path = dl_manager.download(
        'https://storage.googleapis.com/gresearch/ASQA/ASQA.json'
    )

    with tf.io.gfile.GFile(file_path, 'r') as f:
      samples = json.load(f)

    return {
        'train': self._generate_examples(samples['train']),
        'dev': self._generate_examples(samples['dev']),
    }

  def _generate_examples(self, split_samples):
    """Yields examples."""

    def _add_feature(tfds_sample, split_sample, key):
      value = split_sample[key] if split_sample[key] else ''
      tfds_sample[key] = value

    def _iterate_and_add_features_in_dict(
        raw_sample_element, tfds_sample_element, feature_list
    ):
      for element in raw_sample_element:
        tfds_dict = {}
        for feature in feature_list:
          _add_feature(tfds_dict, element, feature)
        tfds_sample_element.append(tfds_dict)

    for sample_id, split_sample in split_samples.items():
      tfds_sample = {
          'sample_id': sample_id,
          'ambiguous_question': split_sample['ambiguous_question'],
          'qa_pairs': [],
          'wikipages': [],
          'annotations': [],
      }

      # Add QA pairs.
      _iterate_and_add_features_in_dict(
          raw_sample_element=split_sample['qa_pairs'],
          tfds_sample_element=tfds_sample['qa_pairs'],
          feature_list=['question', 'short_answers', 'context', 'wikipage'],
      )

      # Add Wikipages.
      for _, wikipage_info in split_sample['wikipages'].items():
        tfds_wikipage = {}
        _add_feature(tfds_wikipage, wikipage_info, 'title')
        _add_feature(tfds_wikipage, wikipage_info, 'url')
        tfds_sample['wikipages'].append(tfds_wikipage)

      # Add annotations.
      for annotation in split_sample['annotations']:
        tfds_annotation = {}
        _add_feature(tfds_annotation, annotation, 'long_answer')
        tfds_annotation['knowledge'] = []
        _iterate_and_add_features_in_dict(
            raw_sample_element=annotation['knowledge'],
            tfds_sample_element=tfds_annotation['knowledge'],
            feature_list=['content', 'wikipage'],
        )
        tfds_sample['annotations'].append(tfds_annotation)

      yield sample_id, tfds_sample



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r"""`tfds build_croissant` command.

Example usage:
```
tfds build_croissant \
  --jsonld=/tmp/croissant.json \
  --data_dir=/tmp/foo \
  --file_format=array_record \
  --record_sets=record1,record2 \
  --mapping='{"document.csv": "~/Downloads/document.csv"}"'
```
"""

import dataclasses
import functools
import json

from etils import epath
import mlcroissant as mlc
import simple_parsing
from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.core.dataset_builders import croissant_builder
from tensorflow_datasets.core.utils import croissant_utils
from tensorflow_datasets.core.utils import version as version_lib
from tensorflow_datasets.scripts.cli import cli_utils


@dataclasses.dataclass(frozen=True, kw_only=True)
class CmdArgs(simple_parsing.helpers.FrozenSerializable, cli_utils.Args):
  """Prepares a Croissant dataset.

  Attributes:
    jsonld: Path to the JSONLD file.
    data_dir: Path where the converted dataset will be stored.
    file_format: File format to convert the dataset to.
    record_sets: The names of the record sets to generate. Each record set will
      correspond to a separate config. If not specified, it will use all the
      record sets.
    mapping: Mapping filename->filepath as a Python dict[str, str] to handle
      manual downloads. If `document.csv` is the FileObject and you downloaded
      it to `~/Downloads/document.csv`, you can specify
      `--mapping='{"document.csv": "~/Downloads/document.csv"}'`
    download_dir: Where to place downloads. Default to `<data_dir>/downloads/`.
    publish_dir: Where to optionally publish the dataset after it has been
      generated successfully. Should be the root data dir under which datasets
      are stored. If unspecified, dataset will not be published.
    skip_if_published: If the dataset with the same version and config is
      already published, then it will not be regenerated.
    overwrite: Delete pre-existing dataset if it exists.
    overwrite_version: Semantic version of the dataset to be set.
  """

  jsonld: epath.Path
  data_dir: epath.Path
  # Need to override the default use of `Enum.name` for choice options.
  file_format: str = simple_parsing.choice(
      *(file_format.value for file_format in file_adapters.FileFormat),
      default=file_adapters.FileFormat.ARRAY_RECORD.value,
  )
  # Need to manually parse comma-separated list of values, see:
  # https://github.com/lebrice/SimpleParsing/issues/142.
  record_sets: list[str] = simple_parsing.field(
      default_factory=list,
      type=lambda record_sets_str: record_sets_str.split(','),
      nargs='?',
  )
  mapping: str | None = None
  download_dir: epath.Path | None = None
  publish_dir: epath.Path | None = None
  skip_if_published: bool = False
  overwrite: bool = False
  overwrite_version: str | None = None

  @functools.cached_property
  def mapping_json(self) -> dict[str, epath.PathLike]:
    if self.mapping:
      try:
        return json.loads(self.mapping)
      except json.JSONDecodeError as e:
        raise ValueError(
            f'Error parsing mapping parameter: {self.mapping}'
        ) from e
    return {}

  @functools.cached_property
  def dataset(self) -> mlc.Dataset:
    return mlc.Dataset(jsonld=self.jsonld, mapping=self.mapping_json)

  @functools.cached_property
  def dataset_name(self) -> str:
    return croissant_utils.get_dataset_name(self.dataset)

  @functools.cached_property
  def record_set_ids(self) -> list[str]:
    return self.record_sets or croissant_utils.get_record_set_ids(
        self.dataset.metadata
    )

  @functools.cached_property
  def version(self) -> version_lib.Version:
    # In TFDS, version is a mandatory attribute, while in Croissant it is only a
    # recommended attribute. If the version is unspecified in Croissant, we set
    # it to `1.0.0` in TFDS.
    return version_lib.Version(
        self.overwrite_version or self.dataset.metadata.version or '1.0.0'
    )

  def execute(self) -> None:
    """Creates Croissant Builders and prepares them."""
    for record_set_id in self.record_set_ids:
      prepare_croissant_builder(args=self, record_set_id=record_set_id)


def prepare_croissant_builder(
    args: CmdArgs, record_set_id: str
) -> croissant_builder.CroissantBuilder:
  """Returns prepared Croissant Builder for the given record set id.

  Args:
    args: CLI arguments.
    record_set_id: Record set id.
  """
  builder = croissant_builder.CroissantBuilder(
      jsonld=args.jsonld,
      record_set_ids=[record_set_id],
      file_format=args.file_format,
      data_dir=args.data_dir,
      mapping=args.mapping_json,
      overwrite_version=args.version,
  )
  cli_utils.download_and_prepare(
      builder=builder,
      download_config=None,
      download_dir=args.download_dir,
      publish_dir=args.publish_dir,
      skip_if_published=args.skip_if_published,
      overwrite=args.overwrite,
      beam_pipeline_options=None,
  )
  return builder



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SpeechCommands dataset."""

import os

import numpy as np
from tensorflow_datasets.core import lazy_imports_lib
import tensorflow_datasets.public_api as tfds

_DOWNLOAD_PATH = (
    'http://download.tensorflow.org/data/speech_commands_v0.02.tar.gz'
)
_TEST_DOWNLOAD_PATH_ = (
    'http://download.tensorflow.org/data/speech_commands_test_set_v0.02.tar.gz'
)

_SPLITS = ['train', 'valid', 'test']

WORDS = ['down', 'go', 'left', 'no', 'off', 'on', 'right', 'stop', 'up', 'yes']
SILENCE = '_silence_'
UNKNOWN = '_unknown_'
BACKGROUND_NOISE = '_background_noise_'
SAMPLE_RATE = 16000


class Builder(tfds.core.GeneratorBasedBuilder):
  """The Speech Commands dataset for keyword detection."""

  VERSION = tfds.core.Version('0.0.3')
  RELEASE_NOTES = {
      '0.0.3': 'Fix audio data type with dtype=tf.int16.',
  }

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'audio': tfds.features.Audio(
                file_format='wav', sample_rate=SAMPLE_RATE, dtype=np.int16
            ),
            'label': tfds.features.ClassLabel(names=WORDS + [SILENCE, UNKNOWN]),
        }),
        supervised_keys=('audio', 'label'),
        # Homepage of the dataset for documentation
        homepage='https://arxiv.org/abs/1804.03209',
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""

    dl_path, dl_test_path = dl_manager.download(
        [_DOWNLOAD_PATH, _TEST_DOWNLOAD_PATH_]
    )

    train_paths, validation_paths = self._split_archive(
        dl_manager.iter_archive(dl_path)
    )

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                'archive': dl_manager.iter_archive(dl_path),
                'file_list': train_paths,
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={
                'archive': dl_manager.iter_archive(dl_path),
                'file_list': validation_paths,
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                'archive': dl_manager.iter_archive(dl_test_path),
                'file_list': None,
            },
        ),
    ]

  def _generate_examples(self, archive, file_list):
    """Yields examples."""
    for path, file_obj in archive:
      if file_list is not None and path not in file_list:
        continue
      relpath, wavname = os.path.split(path)
      _, word = os.path.split(relpath)
      example_id = '{}_{}'.format(word, wavname)
      if word in WORDS:
        label = word
      elif word == SILENCE or word == BACKGROUND_NOISE:
        # The main tar file already contains all of the test files, except for
        # the silence ones. In fact it does not contain silence files at all.
        # So for the test set we take the silence files from the test tar file,
        # while for train and validation we build them from the
        # _background_noise_ folder.
        label = SILENCE
      else:
        # Note that in the train and validation there are a lot more _unknown_
        # labels than any of the other ones.
        label = UNKNOWN

      if word == BACKGROUND_NOISE:
        # Special handling of background noise. We need to cut these files to
        # many small files with 1 seconds length, and transform it to silence.
        audio_samples = np.array(
            lazy_imports_lib.lazy_imports.pydub.AudioSegment.from_file(
                file_obj, format='wav'
            ).get_array_of_samples()
        )

        for start in range(
            0, len(audio_samples) - SAMPLE_RATE, SAMPLE_RATE // 2
        ):
          audio_segment = audio_samples[start : start + SAMPLE_RATE]
          cur_id = '{}_{}'.format(example_id, start)
          example = {'audio': audio_segment, 'label': label}
          yield cur_id, example
      else:
        try:
          example = {
              'audio': np.array(
                  lazy_imports_lib.lazy_imports.pydub.AudioSegment.from_file(
                      file_obj, format='wav'
                  ).get_array_of_samples()
              ),
              'label': label,
          }
          yield example_id, example
        except (
            lazy_imports_lib.lazy_imports.pydub.exceptions.CouldntDecodeError
        ):
          pass

  def _split_archive(self, train_archive):
    train_paths = []
    for path, file_obj in train_archive:
      if 'testing_list.txt' in path:
        train_test_paths = file_obj.read().strip().splitlines()
        train_test_paths = [p.decode('ascii') for p in train_test_paths]
      elif 'validation_list.txt' in path:
        validation_paths = file_obj.read().strip().splitlines()
        validation_paths = [p.decode('ascii') for p in validation_paths]
      elif path.endswith('.wav'):
        train_paths.append(path)

    # Original validation files did include silence - we add them manually here
    validation_paths.append(os.path.join(BACKGROUND_NOISE, 'running_tap.wav'))

    # The paths for the train set is just whichever paths that do not exist in
    # either the test or validation splits.
    train_paths = (
        set(train_paths) - set(validation_paths) - set(train_test_paths)
    )

    return train_paths, validation_paths



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for wake_vision dataset module."""

from tensorflow_datasets import testing
from tensorflow_datasets.datasets.wake_vision import wake_vision_dataset_builder


class WakeVisionTest(testing.DatasetBuilderTestCase):
  """Tests for wake_vision dataset."""

  DATASET_CLASS = wake_vision_dataset_builder.Builder
  SPLITS = {
      'train_large': 16,
      'train_quality': 4,
      'validation': 11,
      'test': 10,
  }

  DL_EXTRACT_RESULT = {
      'train_images': [
          'wake-vision-train-dummy-1.tar.gz',
          'wake-vision-train-dummy-2.tar.gz',
      ],
      'validation_images': ['wake-vision-validation-dummy.tar.gz'],
      'test_images': ['wake-vision-test-dummy.tar.gz'],
      'train_large_metadata': 'wake_vision_train_large.csv',
      'train_quality_metadata': 'wake_vision_train_quality.csv',
      'validation_metadata': 'wake_vision_validation.csv',
      'test_metadata': 'wake_vision_test.csv',
  }


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Lazy import utils.

Please, use `etils.epy.lazy_imports` for lazy imports.
"""

from __future__ import annotations

import importlib

from etils import epy


def tf_error_callback(module_name: Exception):
  del module_name
  print("\n\n***************************************************************")
  print(
      "Failed to import TensorFlow. Please note that TensorFlow is not "
      "installed by default when you install TFDS. This allows you "
      "to choose to install either `tf-nightly` or `tensorflow`. "
      "Please install the most recent version of TensorFlow, by "
      "following instructions at https://tensorflow.org/install."
  )
  print("***************************************************************\n\n")


MIN_TF_VERSION = "2.1.0"

_ensure_tf_version_called = False


# Ensure TensorFlow version is sufficiently recent. This needs to happen after
# TensorFlow was imported, as it is referenced.
def ensure_tf_version(module_name: str):
  """Ensures TensorFlow version is sufficient."""
  # Only check the first time.
  global _ensure_tf_version_called
  if _ensure_tf_version_called:
    return
  _ensure_tf_version_called = True

  tf = importlib.import_module(module_name)
  tf_version = tf.__version__  # pytype: disable=attribute-error
  if tf_version < MIN_TF_VERSION:
    raise ImportError(
        "This version of TensorFlow Datasets requires TensorFlow "
        f"version >= {MIN_TF_VERSION}; Detected an installation of version "
        f"{tf_version}. Please upgrade TensorFlow to proceed."
    )


def array_record_error_callback(module_name: Exception):
  del module_name
  print("\n\n***************************************************************")
  print(
      "Failed to import ArrayRecord. This probably means that you are running"
      " on macOS or Windows. ArrayRecord currently does not work for your"
      " infrastructure, because it uses Python bindings in C++. We are actively"
      " working on this issue. Thanks for your understanding."
  )
  print("***************************************************************\n\n")


def mlcroissant_error_callback(module_name: Exception):
  """Error callback for mlcroissant."""
  del module_name
  print("\n\n***************************************************************")
  print("Failed to import mlcroissant.")
  print('Please install mlcroissant using `pip install mlcroissant`.')
  print("***************************************************************\n\n")


def tf_agents_error_callback(module_name: Exception):
  del module_name
  print("\n\n***************************************************************")
  print("Failed to import tf_agents.")
  print('Please install tf_agents using `pip install tf_agents`.')
  print("***************************************************************\n\n")


def datasets_error_callback(module_name: Exception):
  """Error callback for datasets."""
  del module_name
  print("\n\n***************************************************************")
  print("Failed to import datasets.")
  print('Please install datasets using `pip install datasets`.')
  print("***************************************************************\n\n")


# pylint: disable=g-import-not-at-top,unused-import

with epy.lazy_imports(error_callback=mlcroissant_error_callback):
  import mlcroissant  # pytype: disable=import-error


with epy.lazy_imports():
  import apache_beam
  import pandas
  import psutil
  import pyarrow
  import promise
  from pyarrow import parquet
  import requests
  import tree


with epy.lazy_imports(
    error_callback=tf_error_callback, success_callback=ensure_tf_version
):
  import tensorflow  # pytype: disable=import-error

with epy.lazy_imports(error_callback=tf_agents_error_callback):
  import tf_agents  # pytype: disable=import-error

with epy.lazy_imports(error_callback=datasets_error_callback):
  import datasets  # pytype: disable=import-error
  import huggingface_hub  # pytype: disable=import-error

with epy.lazy_imports(error_callback=array_record_error_callback):
  from array_record.python import array_record_data_source
  from array_record.python import array_record_module



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mlcroissant as mlc
import pytest
from tensorflow_datasets.core.utils import croissant_utils


@pytest.mark.parametrize(
    'croissant_name,croissant_url,tfds_name',
    [
        (
            'Name+1',
            'https://huggingface.co/datasets/HuggingFaceH4/ultrachat_200k',
            'huggingfaceh4__ultrachat_200k',
        ),
        ('Name+1', 'bad_url', 'name_1'),
        ('Name+1', None, 'name_1'),
    ],
)
def test_get_tfds_dataset_name(croissant_name, croissant_url, tfds_name):
  metadata = mlc.Metadata(name=croissant_name, url=croissant_url)
  dataset = mlc.Dataset.from_metadata(metadata)
  assert (
      croissant_utils.get_tfds_dataset_name(dataset) == tfds_name
  ), f'Expected TFDS name: {tfds_name}'


@pytest.mark.parametrize(
    'attribute,language,expected_text',
    [
        ({'en': 'English Text', 'fr': 'Texte Français'}, None, 'English Text'),
        (
            {'de': 'Deutscher Text', 'fr': 'Texte Français'},
            None,
            'Deutscher Text',
        ),
        (
            {'en': 'English Text', 'fr': 'Texte Français'},
            'fr',
            'Texte Français',
        ),
        ('Simple Text', None, 'Simple Text'),
        ('Simple Text', 'en', 'Simple Text'),
        (None, None, None),
    ],
)
def test_extract_localized_string(attribute, language, expected_text):
  assert (
      croissant_utils.extract_localized_string(attribute, language=language)
      == expected_text
  )


def test_extract_localized_string_raises():
  # Language not found.
  with pytest.raises(
      ValueError,
      match=r"Language 'de' not found in text field keys:",
  ):
    croissant_utils.extract_localized_string(
        {'en': 'English Text', 'fr': 'Texte Français'}, language='de'
    )

  # Empty dictionary.
  with pytest.raises(
      ValueError, match='Dataset `text field` dictionary is empty'
  ):
    croissant_utils.extract_localized_string({}, language=None)

  # Incorrect type.
  with pytest.raises(TypeError, match='must be a string, dictionary, or None'):
    croissant_utils.extract_localized_string(123)


@pytest.mark.parametrize(
    'croissant_name,language,expected_name',
    [
        ({'en': 'English Name', 'fr': 'Nom Français'}, None, 'English Name'),
        (
            {'de': 'Deutscher Name', 'fr': 'Nom Français'},
            None,
            'Deutscher Name',
        ),
        ({'en': 'English Name', 'fr': 'Nom Français'}, 'fr', 'Nom Français'),
        ('Simple Name', None, 'Simple Name'),
    ],
)
def test_get_dataset_name(croissant_name, language, expected_name):
  ctx = mlc.Context(conforms_to='http://mlcommons.org/croissant/1.1')
  metadata = mlc.Metadata(name=croissant_name, ctx=ctx, url=None)
  dataset = mlc.Dataset.from_metadata(metadata)
  assert (
      croissant_utils.get_dataset_name(dataset, language=language)
      == expected_name
  )


def test_get_dataset_name_raises():
  ctx = mlc.Context(conforms_to='http://mlcommons.org/croissant/1.1')
  # Test language not found in name.
  metadata_lang_not_found = mlc.Metadata(
      name={'en': 'English Name', 'fr': 'Nom Français'}, ctx=ctx, url=None
  )
  dataset_lang_not_found = mlc.Dataset.from_metadata(metadata_lang_not_found)
  with pytest.raises(
      ValueError, match=r"Language 'de' not found in name keys:"
  ):
    croissant_utils.get_dataset_name(dataset_lang_not_found, language='de')

  # Test empty dictionary name.
  metadata_empty_dict = mlc.Metadata(name={}, ctx=ctx, url=None)
  dataset_empty_dict = mlc.Dataset.from_metadata(metadata_empty_dict)
  with pytest.raises(ValueError, match='Dataset `name` dictionary is empty.'):
    croissant_utils.get_dataset_name(dataset_empty_dict, language=None)


def test_get_dataset_name_url_precedence():
  ctx = mlc.Context(conforms_to='http://mlcommons.org/croissant/1.1')
  # Test that URL prefix removal works and takes precedence over name.
  metadata = mlc.Metadata(
      name='Should Be Ignored',
      ctx=ctx,
      url='https://huggingface.co/datasets/user/dataset_name',
  )
  dataset = mlc.Dataset.from_metadata(metadata)
  assert croissant_utils.get_dataset_name(dataset) == 'user/dataset_name'

  # Test that URL precedence also works when the name is a dict.
  metadata_dict_name = mlc.Metadata(
      name={'en': 'Should Be Ignored'},
      ctx=ctx,
      url='https://huggingface.co/datasets/another/other_dataset',
  )
  dataset_dict_name = mlc.Dataset.from_metadata(metadata_dict_name)
  assert (
      croissant_utils.get_dataset_name(dataset_dict_name)
      == 'another/other_dataset'
  )

  # Test that non-HuggingFace URLs don't cause name to be ignored.
  metadata_other_url = mlc.Metadata(
      name='Not Ignored',
      ctx=ctx,
      url='https://example.com/dataset',
  )
  dataset_other_url = mlc.Dataset.from_metadata(metadata_other_url)
  assert croissant_utils.get_dataset_name(dataset_other_url) == 'Not Ignored'


@pytest.mark.parametrize(
    'croissant_version,tfds_version',
    [
        ('1.0', '1.0.0'),
        ('1.2', '1.2.0'),
        ('1.2.3', '1.2.3'),
        ('1.2.3.4', '1.2.3.4'),
        (None, None),
    ],
)
def test_get_croissant_version(croissant_version, tfds_version):
  assert (
      croissant_utils.get_croissant_version(croissant_version) == tfds_version
  )


def test_get_record_set_ids():
  metadata = mlc.Metadata(
      name='dummy_dataset',
      url='https://dummy_url',
      record_sets=[
          mlc.RecordSet(
              id='record_set_1',
              fields=[],
          ),
          mlc.RecordSet(
              id='record_set_2',
              data_types=['http://mlcommons.org/croissant/Split'],
              fields=[
                  mlc.Field(
                      id='record_set_2/name', data_types=mlc.DataType.TEXT
                  )
              ],
              data=[
                  {'record_set_2/name': 'train'},
                  {'record_set_2/name': 'test'},
              ],
          ),
      ],
  )
  record_set_ids = croissant_utils.get_record_set_ids(metadata=metadata)
  assert record_set_ids == ['record_set_1']


def test_get_split_recordset():
  record_sets = [
      mlc.RecordSet(
          id='records',
          fields=[
              mlc.Field(
                  id='records/split',
                  data_types=[mlc.DataType.TEXT],
                  source=mlc.Source(field='splits/name'),
                  references=mlc.Source(field='splits/name'),
              )
          ],
      ),
      mlc.RecordSet(
          id='splits',
          key='splits/name',
          data_types=[mlc.DataType.SPLIT],
          fields=[
              mlc.Field(
                  id='splits/name', name='name', data_types=mlc.DataType.TEXT
              )
          ],
          data=[{'splits/name': 'train'}, {'splits/name': 'test'}],
      ),
  ]
  metadata = mlc.Metadata(name='dummy', url='dum.my', record_sets=record_sets)
  dataset = mlc.Dataset.from_metadata(metadata)
  split_recordset = croissant_utils.get_split_recordset(
      record_set=dataset.metadata.record_sets[0], metadata=metadata
  )
  assert split_recordset is not None
  assert split_recordset.reference_field.id == 'records/split'
  assert split_recordset.split_record_set.id == 'splits'


def test_get_split_recordset_with_no_split_recordset():
  record_sets = [
      mlc.RecordSet(
          id='labels',
          key='labels/label',
          fields=[
              mlc.Field(
                  id='labels/label',
                  name='label',
                  data_types=mlc.DataType.TEXT,
              )
          ],
          data=[{'labels/label': 'bird'}, {'labels/label': 'bike'}],
      ),
      mlc.RecordSet(
          id='samples',
          fields=[
              mlc.Field(
                  id='samples/label',
                  data_types=mlc.DataType.TEXT,
                  source=mlc.Source(field='labels/label'),
                  references=mlc.Source(field='labels/label'),
              )
          ],
      ),
  ]
  metadata = mlc.Metadata(name='dummy', url='dum.my', record_sets=record_sets)
  split_recordset = croissant_utils.get_split_recordset(
      record_set=metadata.record_sets[0], metadata=metadata
  )
  assert split_recordset is None



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""PatchCamelyon images dataset."""

import tensorflow_datasets.public_api as tfds

_URL = 'https://patchcamelyon.grand-challenge.org/'


class Builder(tfds.core.GeneratorBasedBuilder):
  """PatchCamelyon."""

  VERSION = tfds.core.Version('2.0.0')
  RELEASE_NOTES = {
      '2.0.0': 'New split API (https://tensorflow.org/datasets/splits)',
  }

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'id': tfds.features.Text(),
            'image': tfds.features.Image(
                shape=(96, 96, 3), encoding_format='png'
            ),
            'label': tfds.features.ClassLabel(num_classes=2),
        }),
        supervised_keys=('image', 'label'),
        homepage=_URL,
    )

  def _split_generators(self, dl_manager):
    base_url = 'https://zenodo.org/record/2546921/files/'
    resources = {
        'test_x': base_url + 'camelyonpatch_level_2_split_test_x.h5.gz',
        'test_y': base_url + 'camelyonpatch_level_2_split_test_y.h5.gz',
        'train_x': base_url + 'camelyonpatch_level_2_split_train_x.h5.gz',
        'train_y': base_url + 'camelyonpatch_level_2_split_train_y.h5.gz',
        'valid_x': base_url + 'camelyonpatch_level_2_split_valid_x.h5.gz',
        'valid_y': base_url + 'camelyonpatch_level_2_split_valid_y.h5.gz',
    }
    paths = dl_manager.download_and_extract(resources)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST, gen_kwargs=dict(split='test', paths=paths)
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN, gen_kwargs=dict(split='train', paths=paths)
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs=dict(split='valid', paths=paths),
        ),
    ]

  def _generate_examples(self, split, paths):
    """Generates images and labels given the image directory path.

    Args:
      split: name of the split to generate examples for (test, train, valid).
      paths: dictionary with the paths to the h5 files for each split.

    Yields:
      A dictionary with the image and the corresponding label.
    """
    h5py = tfds.core.lazy_imports.h5py

    filepath_x = paths[split + '_x']
    filepath_y = paths[split + '_y']
    with h5py.File(filepath_x, 'r') as f_x, h5py.File(filepath_y, 'r') as f_y:
      images = f_x['x']
      labels = f_y['y']  # Note: Labels are in a N x 1 x 1 x 1 tensor.

      for i, (image, label) in enumerate(zip(images, labels)):
        label = label.flatten()[0]
        id_ = '%s_%d' % (split, i)
        record = {'id': id_, 'image': image, 'label': label}
        yield id_, record



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SCAN tasks with various different splits."""

import json
import os

from etils import epath
import tensorflow_datasets.public_api as tfds

_DATA_URL = 'https://github.com/brendenlake/SCAN/archive/master.zip'
_MCD_SPLITS_URL = (
    'https://storage.googleapis.com/cfq_dataset/scan-splits.tar.gz'
)


class ScanConfig(tfds.core.BuilderConfig):
  """BuilderConfig for SCAN.

  Splits can be read in two formats:

  1) As a pair of train and test files where each file contains one example
     input and output per line.
  2) With a 'splitfile' which contains for each split the indices into the full
     (unsplit) dataset.
  """

  def __init__(self, *, name, directory=None, splitfile=None, **kwargs):
    """BuilderConfig for SCAN.

    Args:
      name: Unique name of the split.
      directory: Which subdirectory to read the data files from.
      splitfile: If set the samples are read from the original archive
        (tasks.txt) but the splits are created using this index file.
      **kwargs: keyword arguments forwarded to super.
    """
    # Version history:
    super(ScanConfig, self).__init__(
        name=name, version=tfds.core.Version('1.1.1'), **kwargs
    )
    self.splitfile = splitfile
    if 'mcd' in name:
      self.splitfile = name + '.json'
    if self.splitfile and directory is None:
      self.directory = ''
    elif directory is None:
      self.directory = name + '_split'
    else:
      self.directory = directory


_COMMANDS = 'commands'
_ACTIONS = 'actions'


class Builder(tfds.core.GeneratorBasedBuilder):
  """SCAN task / splits as proposed by Brenden M. Lake and Marco Baroni."""

  BUILDER_CONFIGS = [
      ScanConfig(name='simple'),
      ScanConfig(name='addprim_jump', directory='add_prim_split'),
      ScanConfig(name='addprim_turn_left', directory='add_prim_split'),
      ScanConfig(name='filler_num0', directory='filler_split'),
      ScanConfig(name='filler_num1', directory='filler_split'),
      ScanConfig(name='filler_num2', directory='filler_split'),
      ScanConfig(name='filler_num3', directory='filler_split'),
      ScanConfig(name='length'),
      ScanConfig(name='template_around_right', directory='template_split'),
      ScanConfig(name='template_jump_around_right', directory='template_split'),
      ScanConfig(name='template_opposite_right', directory='template_split'),
      ScanConfig(name='template_right', directory='template_split'),
      ScanConfig(name='mcd1'),
      ScanConfig(name='mcd2'),
      ScanConfig(name='mcd3'),
  ]

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            _COMMANDS: tfds.features.Text(),
            _ACTIONS: tfds.features.Text(),
        }),
        supervised_keys=(_COMMANDS, _ACTIONS),
        homepage='https://github.com/brendenlake/SCAN',
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    data_dir = dl_manager.download_and_extract(_DATA_URL)
    data_dir = os.path.join(
        data_dir, 'SCAN-master', self.builder_config.directory
    )
    split = self.builder_config.name
    splitfile = self.builder_config.splitfile
    if 'mcd' in split:
      split_dir = dl_manager.download_and_extract(_MCD_SPLITS_URL)
      split_dir = os.path.join(split_dir, 'scan-splits')
      splitfile = os.path.join(split_dir, splitfile)
    if splitfile:
      kwargs = {
          'datapath': os.path.join(data_dir, 'tasks.txt'),
          'splitpath': splitfile,
      }
      train_kwargs = kwargs.copy()
      train_kwargs['splitname'] = 'train'
      test_kwargs = kwargs.copy()
      test_kwargs['splitname'] = 'test'
    else:
      train_kwargs = {
          'datapath': os.path.join(data_dir, 'tasks_train_' + split + '.txt')
      }
      test_kwargs = {
          'datapath': os.path.join(data_dir, 'tasks_test_' + split + '.txt')
      }
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN, gen_kwargs=train_kwargs
        ),
        tfds.core.SplitGenerator(name=tfds.Split.TEST, gen_kwargs=test_kwargs),
    ]

  def _read_examples(self, datapath):
    with epath.Path(datapath).open() as infile:
      for i, line in enumerate(infile):
        if not line.startswith('IN: '):
          continue
        # Chop the prefix and split string between input and output
        commands, actions = line[len('IN: ') :].strip().split(' OUT: ', 1)
        yield i, {_COMMANDS: commands, _ACTIONS: actions}

  def _generate_examples(self, datapath, splitpath=None, splitname=None):
    """Yields examples."""
    if splitpath:
      all_samples = list(self._read_examples(datapath))
      with epath.Path(splitpath).open() as infile:
        split = json.load(infile)
      for idx in split[splitname + 'Idxs']:
        yield all_samples[idx]
    else:
      for example in self._read_examples(datapath):
        yield example



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Stanford dogs dataset."""

import collections
import os
import re
import xml.etree.ElementTree as ET

from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_URL = "http://vision.stanford.edu/aditya86/ImageNetDogs/main.html"

_IMAGES_URL = "http://vision.stanford.edu/aditya86/ImageNetDogs/images.tar"
_SPLIT_URL = "http://vision.stanford.edu/aditya86/ImageNetDogs/lists.tar"
_ANNOTATIONS_URL = (
    "http://vision.stanford.edu/aditya86/ImageNetDogs/annotation.tar"
)
_NAME_RE = re.compile(r"([\w-]*[/\\])*([\w]*.jpg)$")


class Builder(tfds.core.GeneratorBasedBuilder):
  """Stanford Dogs dataset."""

  # Version 0.2.0: Fix non-deterministic label names
  VERSION = tfds.core.Version("0.2.0")

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            # Images are of varying size
            "image": tfds.features.Image(),
            "image/filename": tfds.features.Text(),
            "label": tfds.features.ClassLabel(num_classes=120),
            # Multiple bounding box per image
            "objects": tfds.features.Sequence(
                {
                    "bbox": tfds.features.BBoxFeature(),
                }
            ),
        }),
        supervised_keys=("image", "label"),
        homepage=_URL,
    )

  def _split_generators(self, dl_manager):
    images_path = dl_manager.download(_IMAGES_URL)
    split_path, annotation_path = dl_manager.download_and_extract(
        [_SPLIT_URL, _ANNOTATIONS_URL]
    )
    xml_file_list = collections.defaultdict(str)

    # Parsing the mat file which contains the list of train/test images
    scipy = tfds.core.lazy_imports.scipy

    def parse_mat_file(file_name):
      with tf.io.gfile.GFile(file_name, "rb") as f:
        parsed_mat_arr = scipy.io.loadmat(f, squeeze_me=True)
      file_list = [
          os.path.split(element)[-1] for element in parsed_mat_arr["file_list"]
      ]

      return file_list, parsed_mat_arr

    for fname in tf.io.gfile.listdir(split_path):
      # Train-test split using train_list.mat and test_list.mat
      full_file_name = os.path.join(split_path, fname)

      if "train" in fname:
        train_list, train_mat_arr = parse_mat_file(full_file_name)
        label_names = set(
            [  # Set to remove duplicates
                os.path.split(element)[-2].lower()  # Extract path/label/img.jpg
                for element in train_mat_arr["file_list"]
            ]
        )
      elif "test" in fname:
        test_list, _ = parse_mat_file(full_file_name)

    self.info.features["label"].names = sorted(label_names)

    for root, _, files in tf.io.gfile.walk(annotation_path):
      # Parsing the XML file which have the image annotations
      for fname in files:
        annotation_file_name = os.path.join(root, fname)
        with tf.io.gfile.GFile(annotation_file_name, "rb") as f:
          xml_file_list[fname] = ET.parse(f)

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "archive": dl_manager.iter_archive(images_path),
                "file_names": train_list,
                "annotation_files": xml_file_list,
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "archive": dl_manager.iter_archive(images_path),
                "file_names": test_list,
                "annotation_files": xml_file_list,
            },
        ),
    ]

  def _generate_examples(self, archive, file_names, annotation_files):
    """Generate dog images, labels, bbox attributes given the directory path.

    Args:
      archive: object that iterates over the zip file_names : list of train/test
        image file names obtained from mat file annotation_files : dict of image
        file names and their xml object

    Yields:
      Image path, Image file name, its corresponding label and
      bounding box values
    """
    bbox_attrib = ["xmin", "xmax", "ymin", "ymax", "width", "height"]

    for fname, fobj in archive:
      res = _NAME_RE.match(fname)
      if not res or (os.path.split(fname)[-1] not in file_names):
        continue

      label = res.group(1)[:-1].lower()
      file_name = res.group(2)
      attributes = collections.defaultdict(list)
      for element in annotation_files[file_name.split(".")[0]].iter():
        # Extract necessary Bbox attributes from XML file
        if element.tag.strip() in bbox_attrib:
          attributes[element.tag.strip()].append(float(element.text.strip()))

      # BBox attributes in range of 0.0 to 1.0
      def normalize_bbox(bbox_side, image_side):
        return min(bbox_side / image_side, 1.0)

      def build_box(attributes, n):
        return tfds.features.BBox(
            ymin=normalize_bbox(attributes["ymin"][n], attributes["height"][0]),
            xmin=normalize_bbox(attributes["xmin"][n], attributes["width"][0]),
            ymax=normalize_bbox(attributes["ymax"][n], attributes["height"][0]),
            xmax=normalize_bbox(attributes["xmax"][n], attributes["width"][0]),
        )

      yield fname, {
          "image": fobj,
          "image/filename": fname,
          "label": label,
          "objects": [
              {"bbox": build_box(attributes, n)}
              for n in range(len(attributes["xmin"]))
          ],
      }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ImageNet-Sketch dataset."""

from tensorflow_datasets.datasets.imagenet2012 import imagenet_common
import tensorflow_datasets.public_api as tfds

_BASE_URL = 'https://github.com/HaohanWang/ImageNet-Sketch'
_IMAGENET_SKETCH_URL = 'https://drive.google.com/u/0/uc?id=1Mj0i5HBthqH1p_yeXzsg22gZduvgoNeA&export=download'


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for imagenet_sketch dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    imagenet_names_file = imagenet_common.label_names_file()
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(),
            'label': tfds.features.ClassLabel(names_file=imagenet_names_file),
            'file_name': tfds.features.Text(),
        }),
        supervised_keys=('image', 'label'),
        homepage=_BASE_URL,
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    path = dl_manager.download(_IMAGENET_SKETCH_URL)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                'archive': dl_manager.iter_archive(path),
            },
        ),
    ]

  def _generate_examples(self, archive):
    """Generate images and labels for splits."""
    for fname, fobj in archive:
      assert fname.startswith('sketch/') and fname.endswith('.JPEG')
      fname = fname[len('sketch/') :]
      yield fname, {'image': fobj, 'file_name': fname, 'label': fname[:9]}



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ClassLabel feature."""

from collections.abc import Iterable
from typing import Optional, Union

from etils import epath
import numpy as np
from tensorflow_datasets.core.features import feature as feature_lib
from tensorflow_datasets.core.features import tensor_feature
from tensorflow_datasets.core.proto import feature_pb2
from tensorflow_datasets.core.utils import type_utils

Json = type_utils.Json


class ClassLabel(tensor_feature.Tensor):
  """`FeatureConnector` for integer class labels."""

  # If updating the signature here, LabeledImage should likely be updated too.
  def __init__(
      self,
      *,
      num_classes: int | None = None,
      names: Iterable[str] | None = None,
      names_file: epath.PathLike | None = None,
      doc: feature_lib.DocArg = None,
  ):
    """Constructs a ClassLabel FeatureConnector.

    There are 3 ways to define a ClassLabel, which correspond to the 3
    arguments:

     * `num_classes`: create 0 to (num_classes-1) labels
     * `names`: a list of label strings
     * `names_file`: a file containing the list of labels.

    Note: On python2, the strings are encoded as utf-8.

    Args:
      num_classes: Number of classes. All labels must be < num_classes.
      names: String names for the integer classes. The order in which the names
        are provided is kept.
      names_file: Path to a file with names for the integer classes, one per
        line.
      doc: Documentation of this feature (e.g. description).
    """
    super(ClassLabel, self).__init__(shape=(), dtype=np.int64, doc=doc)

    self._num_classes: Optional[int] = None
    self._str2int: Optional[dict[str, int]] = None
    self._int2str: Optional[list[str]] = None

    # The label is explicitly set as undefined (no label defined)
    if all(a is None for a in (num_classes, names, names_file)):
      return

    if sum(a is not None for a in (num_classes, names, names_file)) != 1:
      raise ValueError(
          "Only a single argument of ClassLabel() should be provided."
      )

    if num_classes is not None:
      self._num_classes = num_classes
    elif names is not None:
      self.names = list(names)
    elif names_file is not None:
      self.names = _load_names_from_file(epath.Path(names_file))

  @property
  def num_classes(self) -> Optional[int]:
    return self._num_classes

  @property
  def names(self) -> list[str]:
    if not self._int2str:
      return [str(i) for i in range(self._num_classes)]
    return self._int2str

  @names.setter
  def names(self, new_names: list[str]):
    int2str = new_names
    # Names can only be defined once
    if self._int2str is not None and self._int2str != int2str:
      raise ValueError(
          "Trying to overwrite already defined ClassLabel names. Previous: {} "
          ", new: {}".format(self._int2str, int2str)
      )

    # Set-up [new] names
    self._int2str = int2str
    self._str2int = {name: i for i, name in enumerate(self._int2str)}
    if len(self._int2str) != len(self._str2int):
      raise ValueError(
          "Some label names are duplicated. Each label name should be unique."
      )

    # If num_classes has been defined, ensure that num_classes and names match
    num_classes = len(self._str2int)
    if self._num_classes is None:
      self._num_classes = num_classes
    elif self._num_classes != num_classes:
      raise ValueError(
          "ClassLabel number of names do not match the defined num_classes. "
          "Got {} names VS {} num_classes".format(
              num_classes, self._num_classes
          )
      )

  def str2int(self, str_value: str) -> int:
    """Conversion class name string => integer."""
    if self._str2int:
      return self._str2int[str_value]

    # No names provided, try to integerize
    failed_parse = False
    try:
      int_value = int(str_value)
    except ValueError:
      failed_parse = True
    if failed_parse or not 0 <= int_value < self._num_classes:
      raise ValueError("Invalid string class label %s" % str_value)
    return int_value

  def int2str(self, int_value: int) -> str:
    """Conversion integer => class name string."""
    if self._int2str:
      # Maybe should support batched np array/eager tensors, to allow things
      # like
      # out_ids = model(inputs)
      # labels = cifar10.info.features['label'].int2str(out_ids)
      return self._int2str[int_value]

    # No names provided, return str(int)
    if not 0 <= int_value < self._num_classes:
      raise ValueError("Invalid integer class label %d" % int_value)
    return str(int_value)

  def encode_example(self, example_data):
    if self._num_classes is None:
      raise ValueError(
          "Trying to use ClassLabel feature with undefined number of class. "
          "Please set ClassLabel.names or num_classes."
      )

    # If a string is given, convert to associated integer
    if isinstance(example_data, str):
      example_data = self.str2int(example_data)
    elif isinstance(example_data, bytes):
      # Accept bytes if user yield `tensor.numpy()`
      # Python 3 doesn't interpret byte strings as strings directly.
      example_data = self.str2int(example_data.decode("utf-8"))

    # Allowing -1 to mean no label.
    if not -1 <= example_data < self._num_classes:
      raise ValueError(
          "Class label %d greater than configured num_classes %d"
          % (example_data, self._num_classes)
      )
    return example_data

  def save_metadata(self, data_dir, feature_name=None) -> None:
    """See base class for details."""
    # Save names if defined
    if self._str2int is not None:
      names_filepath = self.get_names_filepath(data_dir, feature_name)
      _write_names_to_file(names_filepath, self.names)

  def load_metadata(self, data_dir, feature_name=None) -> Optional[list[str]]:
    """See base class for details."""
    # Restore names if defined
    names_filepath = self.get_names_filepath(data_dir, feature_name)
    try:
      self.names = _load_names_from_file(names_filepath)
    except OSError:
      pass

  def _additional_repr_info(self) -> dict[str, int]:
    return {"num_classes": self.num_classes}  # pytype: disable=bad-return-type  # always-use-property-annotation

  def repr_html(self, ex: int) -> str:
    """Class labels are displayed with their name."""
    if ex == -1:
      return "-"
    elif not self._int2str:
      return str(ex)
    else:
      return f"{ex} ({self.int2str(ex)})"

  @classmethod
  def from_json_content(
      cls, value: Union[Json, feature_pb2.ClassLabel]
  ) -> "ClassLabel":
    if isinstance(value, dict):
      return cls(**value)
    return cls(num_classes=value.num_classes)

  def to_json_content(self) -> feature_pb2.ClassLabel:  # pytype: disable=signature-mismatch  # overriding-return-type-checks
    return feature_pb2.ClassLabel(num_classes=self.num_classes)

  @classmethod
  def get_names_filepath(cls, data_dir, feature_name: str) -> epath.Path:
    return epath.Path(data_dir) / f"{feature_name}.labels.txt"


def _load_names_from_file(names_filepath: epath.Path) -> list[str]:
  return [
      name.strip()
      for name in names_filepath.read_text().split("\n")
      if name.strip()  # Filter empty names
  ]


def _write_names_to_file(
    names_filepath: epath.Path, names: Iterable[str]
) -> None:
  names_filepath.write_text("\n".join(names) + "\n")



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Radon dataset."""

from __future__ import absolute_import
from __future__ import annotations
from __future__ import division
from __future__ import print_function

import collections
import urllib

from etils import epath
import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

BASE_URL = 'http://www.stat.columbia.edu/~gelman/arm/examples/radon/'


def convert_to_int(d):
  return np.int32(d)


def convert_to_float(d):
  return np.float32(d)


def return_same(d):
  return d


def features():
  return collections.OrderedDict([
      ('idnum', (tf.int32, convert_to_int)),
      ('state', (tf.string, return_same)),
      ('state2', (tf.string, return_same)),
      ('stfips', (tf.int32, convert_to_int)),
      ('zip', (tf.int32, convert_to_int)),
      ('region', (tf.int32, convert_to_int)),
      ('typebldg', (tf.int32, convert_to_int)),
      ('floor', (tf.int32, convert_to_int)),
      ('room', (tf.int32, convert_to_int)),
      ('basement', (tf.string, return_same)),
      ('windoor', (tf.string, return_same)),
      ('rep', (tf.int32, convert_to_int)),
      ('stratum', (tf.int32, convert_to_int)),
      ('wave', (tf.int32, convert_to_int)),
      ('starttm', (tf.int32, convert_to_int)),
      ('stoptm', (tf.int32, convert_to_int)),
      ('startdt', (tf.int32, convert_to_int)),
      ('stopdt', (tf.int32, convert_to_int)),
      ('pcterr', (tf.float32, convert_to_float)),
      ('adjwt', (tf.float32, convert_to_float)),
      ('dupflag', (tf.int32, convert_to_int)),
      ('zipflag', (tf.int32, convert_to_int)),
      ('cntyfips', (tf.int32, convert_to_int)),
      ('county', (tf.string, return_same)),
      ('Uppm', (tf.float32, convert_to_float)),
      ('lon', (tf.float32, convert_to_float)),
      ('lat', (tf.float32, convert_to_float)),
  ])


class Builder(tfds.core.GeneratorBasedBuilder):
  """Radon dataset."""

  VERSION = tfds.core.Version('1.0.0')

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'activity': np.float32,
            'features': {
                name: dtype for name, (dtype, _) in features().items()
            },
        }),
        supervised_keys=('features', 'activity'),
        homepage='http://www.stat.columbia.edu/~gelman/arm/examples/radon/',
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    paths = dl_manager.download({
        'file_path_srrs2': urllib.parse.urljoin(BASE_URL, 'srrs2.dat'),
        'file_path_cty': urllib.parse.urljoin(BASE_URL, 'cty.dat'),
    })
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs=paths,
        ),
    ]

  def _generate_examples(self, file_path_srrs2, file_path_cty):
    """Yields examples."""
    pd = tfds.core.lazy_imports.pandas

    with epath.Path(file_path_srrs2).open() as f:
      df_srrs2 = pd.read_csv(f)
    with epath.Path(file_path_cty).open() as f:
      df_cty = pd.read_csv(f)

    df_srrs2.rename(columns=str.strip, inplace=True)
    df_cty.rename(columns=str.strip, inplace=True)

    # We will now join datasets on Federal Information Processing Standards
    # (FIPS) id, ie, codes that link geographic units, counties and county
    # equivalents. http://jeffgill.org/Teaching/rpqm_9.pdf
    df_srrs2['fips'] = 1000 * df_srrs2.stfips + df_srrs2.cntyfips
    df_cty['fips'] = 1000 * df_cty.stfips + df_cty.ctfips

    df = df_srrs2.merge(df_cty[['fips', 'Uppm', 'lon', 'lat']], on='fips')
    df = df.drop_duplicates(subset='idnum')
    df.drop('fips', axis=1, inplace=True)

    df['wave'].replace({'  .': '-1'}, inplace=True)
    df['rep'].replace({' .': '-1'}, inplace=True)
    df['zip'].replace({'     ': '-1'}, inplace=True)

    for i, (_, row) in enumerate(df.iterrows()):
      radon_val = row.pop('activity')
      yield i, {
          'activity': float(radon_val),
          'features': {
              name: features()[name][1](value) for name, value in row.items()
          },
      }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Imagenet few-shot datasets."""

import os

from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
from tensorflow_datasets.datasets.imagenet2012 import imagenet_common
from tensorflow_datasets.datasets.imagenet2012_subset import imagenet2012_subset_dataset_builder
import tensorflow_datasets.public_api as tfds

# pylint: disable=line-too-long
SUBSET2FILES = {
    '1shot': tfds.core.gcs_path('downloads/imagenet2012_fewshot/1shot.txt'),
    '5shot': tfds.core.gcs_path('downloads/imagenet2012_fewshot/5shot.txt'),
    '10shot': tfds.core.gcs_path('downloads/imagenet2012_fewshot/10shot.txt'),
}
TUNE_FILE = tfds.core.gcs_path('downloads/imagenet2012_fewshot/tune.txt')


class Builder(imagenet2012_subset_dataset_builder.Builder):
  """Class balanced subset of Imagenet 2012 dataset for few-shot learning."""

  BUILDER_CONFIGS = [
      tfds.core.BuilderConfig(  # pylint: disable=g-complex-comprehension
          name=subset_size,
          description='{} of total ImageNet training set.'.format(subset_size),
          version=tfds.core.Version('5.0.1'),
      )
      for subset_size in SUBSET2FILES
  ]

  def _info(self):
    names_file = imagenet_common.label_names_file()
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(),
            'label': tfds.features.ClassLabel(names_file=names_file),
            'file_name': tfds.features.Text(),  # Eg: 'n15075141_54.JPEG'
        }),
        supervised_keys=('image', 'label'),
        homepage='http://image-net.org/',
    )

  def _split_generators(self, dl_manager):
    train_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_train.tar')
    val_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_val.tar')

    if not tf.io.gfile.exists(train_path) or not tf.io.gfile.exists(val_path):
      raise AssertionError(
          'ImageNet requires manual download of the data. Please download '
          'the train and val set and place them into: {}, {}'.format(
              train_path, val_path
          )
      )

    # Download and load subset file.
    subset_file = SUBSET2FILES[self.builder_config.name]
    if isinstance(subset_file, list):  # it will only be a list during testing,
      subset_file = subset_file[0]  # where the first entry is 1shot.txt.
    subset = set(subset_file.read_text().splitlines())

    # Get the file for tune split.
    tuneset = set(TUNE_FILE.read_text().splitlines())

    return {
        tfds.Split.TRAIN: self._generate_examples(
            archive=dl_manager.iter_archive(train_path), subset=subset
        ),
        tfds.Split('tune'): self._generate_examples(
            archive=dl_manager.iter_archive(train_path), subset=tuneset
        ),
        tfds.Split.VALIDATION: self._generate_examples(
            archive=dl_manager.iter_archive(val_path),
            validation_labels=imagenet_common.get_validation_labels(val_path),
        ),
    }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the PythonDataSource."""

import pickle
from unittest import mock

from tensorflow_datasets.core.data_sources import python
import tree


def getitem(i):
  return i


def test_create_a_python_data_source():
  source = python.PythonDataSource(length=2, getitem=getitem)
  assert len(source) == 2
  assert source[0] == 0
  assert source[1] == 1
  assert source[2] == 2


def test_iterate_on_a_python_data_source():
  source = python.PythonDataSource(length=42, getitem=getitem)
  i = 0
  for i, j in enumerate(iter(source)):
    assert i == j
  assert i == 41


def test_python_data_source_is_pickable():
  source = python.PythonDataSource(length=42, getitem=getitem)
  source = pickle.loads(pickle.dumps(source))
  assert source[0] == 0


def test_tree_map_structure():
  source = python.PythonDataSource(length=3, getitem=getitem)
  func = mock.MagicMock()
  tree.map_structure(func, source)
  calls = [mock.call(0), mock.call(1), mock.call(2)]
  func.assert_has_calls(calls)


def test_repr():
  source = python.PythonDataSource(length=3, getitem=getitem)
  assert repr(source) == 'PythonDataSource(length=3)'



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""bucc dataset."""
import csv
import os

from etils import epath
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_LANGS = ['de', 'fr', 'zh', 'ru']

_DATA_URLS = 'https://comparable.limsi.fr/bucc2018/'


class BuccConfig(tfds.core.BuilderConfig):
  """Configuration Class for Tatoeba."""

  def __init__(self, *, language, **kwargs):
    if language not in _LANGS:
      raise ValueError('language must be one of {}'.format(_LANGS))

    super(BuccConfig, self).__init__(**kwargs)
    self.language = language


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for bucc dataset."""

  BUILDER_CONFIGS = [
      BuccConfig(  # pylint: disable=g-complex-comprehension
          name='bucc_' + language,
          language=language,
      )
      for language in _LANGS
  ]
  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'source_sentence': tfds.features.Text(),
            'target_sentence': tfds.features.Text(),
            'source_id': tfds.features.Text(),
            'target_id': tfds.features.Text(),
        }),
        supervised_keys=None,
        homepage='https://comparable.limsi.fr/bucc2018/',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    bucc_test_path = os.path.join(
        _DATA_URLS,
        'bucc2018-{}-en.training-gold.tar.bz2'.format(
            self.builder_config.language
        ),
    )
    bucc_dev_path = os.path.join(
        _DATA_URLS,
        'bucc2018-{}-en.sample-gold.tar.bz2'.format(
            self.builder_config.language
        ),
    )

    archive = dl_manager.download_and_extract(
        {'bucc_test_dir': bucc_test_path, 'bucc_dev_dir': bucc_dev_path}
    )

    return {
        'validation': self._generate_examples(
            filepath=os.path.join(
                archive['bucc_dev_dir'],
                'bucc2018',
                self.builder_config.language + '-en',
            )
        ),
        'test': self._generate_examples(
            filepath=os.path.join(
                archive['bucc_test_dir'],
                'bucc2018',
                self.builder_config.language + '-en',
            )
        ),
    }

  def _generate_examples(self, filepath):
    """Yields examples."""
    files = sorted(tf.io.gfile.listdir(filepath))
    target_file = '/'
    source_file = '/'
    source_target_file = '/'
    for file in files:
      if file.endswith('en'):
        target_file = os.path.join(filepath, file)
      elif file.endswith('gold'):
        source_target_file = os.path.join(filepath, file)
      else:
        source_file = os.path.join(filepath, file)

    with epath.Path(target_file).open() as f:
      data = csv.reader(f, delimiter='\t')
      target_sentences = list(data)
    with epath.Path(source_file).open() as f:
      data = csv.reader(f, delimiter='\t')
      source_sentences = list(data)
    with epath.Path(source_target_file).open() as f:
      data = csv.reader(f, delimiter='\t')
      source_target_ids = list(data)
    for id_, pair in enumerate(source_target_ids):
      source_id = pair[0]
      target_id = pair[1]
      source_sent = ''
      target_sent = ''
      for i in range(len(source_sentences)):
        if source_sentences[i][0] == source_id:
          source_sent = source_sentences[i][1]
          source_id = source_sentences[i][0]
          break
      for j in range(len(target_sentences)):
        if target_sentences[j][0] == target_id:
          target_sent = target_sentences[j][1]
          target_id = target_sentences[j][0]
          break
      yield id_, {
          'source_sentence': source_sent,
          'target_sentence': target_sent,
          'source_id': source_id,
          'target_id': target_id,
      }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for rds_importer_builder dataset."""

import tensorflow_datasets.public_api as tfds
from tensorflow_datasets.robotics import dataset_importer_builder


class MockRDSDataset(dataset_importer_builder.DatasetImporterBuilder):
  """DatasetBuilder for mock `bridge` dataset."""

  def get_description(self):
    return 'tbd'

  def get_citation(self):
    return 'tbd'

  def get_homepage(self):
    return 'tbd'

  def get_relative_dataset_location(self):
    return 'bridge/1.0.0'


class MockRDSDatasetTestRlds(tfds.testing.DatasetBuilderTestCase):
  """Tests for an rds dataset."""

  DATASET_CLASS = MockRDSDataset
  SPLITS = {'train': 1}

  SKIP_TF1_GRAPH_MODE = True

  @classmethod
  def setUpClass(cls):
    MockRDSDataset._GCS_BUCKET = cls.dummy_data
    super().setUpClass()


if __name__ == '__main__':
  tfds.testing.test_main()



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r"""Config for community datasets."""

from collections.abc import Mapping, Sequence
import dataclasses
import functools
import threading
from typing import Any

from absl import logging
from etils import epath
# Make sure that github paths are registered. This import makes sure that epath
# understands paths that start with github://.
from tensorflow_datasets.core import github_api  # pylint: disable=unused-import
from tensorflow_datasets.core.utils import resource_utils
import toml


@dataclasses.dataclass(frozen=True)
class NamespaceConfig:
  """Configuration for a namespace."""

  paths: Sequence[epath.Path]
  info: str | None = None

  def __post_init__(self):
    if not self.paths:
      raise ValueError('NamespaceConfig must have at least one path.')


def _as_path_list(path_or_paths: str | Sequence[str]) -> Sequence[epath.Path]:
  if isinstance(path_or_paths, str):
    path_or_paths = [path_or_paths]
  return [epath.Path(p) for p in path_or_paths]


def _load_config(config: Mapping[str, Any]) -> NamespaceConfig:
  return NamespaceConfig(
      paths=_as_path_list(config['paths']),
      info=config.get('info', None),
  )


class NamespaceRegistry:
  """What namespaces there are and where to find their datasets.

  The config file should be in `.toml` format and have the follow structure:

  ```toml
  [huggingface]
  paths = 'github://huggingface/datasets/tree/master/datasets'
  [kaggle]
  paths = ['/path/to/datasets/', '/path/to/more_datasets/']
  [tensorflow_graphics]
  paths = 'gs://tensorflow-graphics/datasets'
  ```

  Attributes:
    config_path: path where the config is stored.
    config_per_namespace: mapping from namespace to configuration.
  """

  def __init__(self, config_path: epath.Path):
    self.config_path = config_path
    # Load the namespace config only when it is used.
    self._config_per_namespace: dict[str, NamespaceConfig] = {}
    self._is_config_per_namespace_initialized = False
    # If multiple threads all initialize the config per namespace at the same
    # time, then this will cause an exception. Use a lock to make sure it's
    # initialized only once.
    self._lock = threading.Lock()

  def _initialize_config_per_namespace(self) -> None:
    with self._lock:
      # If this thread had to wait, it might have been initialized in the
      # meantime, so check again whether it's initialized.
      if not self._is_config_per_namespace_initialized:
        logging.info('Loading namespace config from %s', self.config_path)
        raw_config = toml.loads(self.config_path.read_text())
        for namespace, raw_namespace_config in raw_config.items():
          namespace_config = _load_config(config=raw_namespace_config)
          self.add_namespace(namespace, namespace_config)
        self._is_config_per_namespace_initialized = True

  @property
  def config_per_namespace(self) -> Mapping[str, NamespaceConfig]:
    """Returns a mapping from namespace to configuration."""
    if not self._is_config_per_namespace_initialized:
      self._initialize_config_per_namespace()
    return self._config_per_namespace

  def add_namespace(self, namespace: str, config: NamespaceConfig) -> None:
    if namespace in self._config_per_namespace:
      namespaces = ', '.join(sorted(self._config_per_namespace.keys()))
      raise RuntimeError(
          f'NamespaceRegistry({self.config_path},'
          f' initialized={self._is_config_per_namespace_initialized}):'
          f' namespace {namespace} is already defined! Defined namespaces:'
          f' {namespaces}'
      )
    self._config_per_namespace[namespace] = config


COMMUNITY_CONFIG_PATH = 'community-datasets.toml'


@functools.lru_cache(maxsize=1)
def get_community_config() -> NamespaceRegistry:
  return NamespaceRegistry(
      config_path=resource_utils.tfds_path(COMMUNITY_CONFIG_PATH)
  )



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""AI2DCaption dataset."""

import json
import os.path

import tensorflow_datasets.public_api as tfds

LAYOUT_NAMES = [
    'abstract',
    'circular',
    'columns',
    'linear',
    'rows',
    'tree',
    'unspecified',
]

TOPIC_NAMES = ['astronomy', 'biology', 'engineering', 'unspecified']

TYPE_NAMES = [
    'arrow',
    'image',
    'object',
    'relationship',
    'text',
]

CATEGORIES = [
    'imageCaption',
    'imageTitle',
    'interObjectLinkage',
    'intraObjectLabel',
    'intraObjectLinkage',
    'intraObjectRegionLabel',
    'intraObjectTextLinkage',
    'misc',
    'sectionTitle',
    'unspecified',
]

SPLITS = [
    'auditor_llm_training_examples',
    'gpt4v',
    'llava_15',
    'planner_llm_training_examples',
    'test',
]

JSON_URL_TMPL = 'https://huggingface.co/datasets/abhayzala/AI2D-Caption/resolve/main/ai2d_caption_{split}.json?download=true'

IMAGES_URL = 'http://ai2-website.s3.amazonaws.com/data/ai2d-all.zip'


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for AI2DCaption dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(
                shape=(None, None, 3),
                doc=tfds.features.Documentation(
                    desc='The image of the diagram.',
                ),
            ),
            'image_filename': tfds.features.Text(
                doc=tfds.features.Documentation(
                    desc='Image filename. e.g. "1337.png"',
                ),
            ),
            'topic': tfds.features.ClassLabel(names=TOPIC_NAMES),
            'layout': tfds.features.ClassLabel(names=LAYOUT_NAMES),
            'caption': tfds.features.Text(),
            'relationships': tfds.features.Sequence(tfds.features.Text()),
            'entities': tfds.features.Sequence(
                tfds.features.FeaturesDict({
                    'id': tfds.features.Text(),
                    'type': tfds.features.ClassLabel(names=TYPE_NAMES),
                    'label': tfds.features.Text(),
                    'bounds': tfds.features.BBoxFeature(),
                    # Not always specified:
                    'cat': tfds.features.ClassLabel(names=CATEGORIES),
                    'from': tfds.features.Text(),
                    'to': tfds.features.Text(),
                })
            ),
        }),
        supervised_keys=None,
        homepage='https://huggingface.co/datasets/abhayzala/AI2D-Caption',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    paths = {split: JSON_URL_TMPL.format(split=split) for split in SPLITS}
    paths['images'] = IMAGES_URL
    dl_paths = dl_manager.download(paths)

    return {
        split: self._generate_examples(
            split, dl_paths[split], dl_paths['images']
        )
        for split in SPLITS
    }

  def _generate_examples(self, split, json_path, images_path):
    """Yields examples."""
    # Build an images index from JSON:
    json_data = json.loads(json_path.read_text(encoding='utf-8'))
    metadata_by_filename = {}  # Maps from image id/filename to image metadata.
    for image_metadata in json_data:
      metadata_by_filename[image_metadata['image']] = image_metadata
    # Iterate over the images,ß yield the ones present in metadata_by_filename:
    for image_path, file in tfds.download.iter_archive(
        images_path, tfds.download.ExtractMethod.ZIP
    ):
      if not image_path.startswith('ai2d/images/'):
        continue
      image_id = os.path.basename(image_path)
      if (metadata := metadata_by_filename.get(image_id)) is None:
        continue
      # Convert bounding box format from REL_XYXY to TFDS format.
      entities = list(metadata['entities'].values())
      for entity in entities:
        # auditor_llm_training_examples split has non-sense bounds (max<min).
        if (
            bounds := entity.get('bounds')
        ) and split != 'auditor_llm_training_examples':
          xmin, ymin, xmax, ymax = [c / 100.0 for c in bounds]
        else:
          xmin, ymin, xmax, ymax = 0.0, 0.0, 0.0, 0.0
        entity['bounds'] = tfds.features.BBox(ymin, xmin, ymax, xmax)
        entity.setdefault('label', '')
        entity.setdefault('cat', 'unspecified')
        entity.setdefault('from', '')
        entity.setdefault('to', '')
      relationships = metadata.get('relationships', [])
      # ai2d_caption_test.json has a few relationships expressed as a dict.
      if isinstance(relationships, dict):
        relationships = list(relationships.values())
      yield image_id, {
          'image_filename': image_id,
          'image': file,
          'topic': metadata.get('topic', 'unspecified'),
          # layout may be an empty string, hence the following construct.
          'layout': metadata.get('layout', None) or 'unspecified',
          'caption': metadata.get('caption', ''),
          'relationships': relationships,
          'entities': entities,
      }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""dSprites dataset."""

import numpy as np
import tensorflow_datasets.public_api as tfds

_URL = (
    "https://github.com/deepmind/dsprites-dataset/blob/master/"
    "dsprites_ndarray_co1sh3sc6or40x32y32_64x64.hdf5?raw=true"
)


class Builder(tfds.core.GeneratorBasedBuilder):
  """dSprites data set."""

  VERSION = tfds.core.Version("2.0.0")
  SUPPORTED_VERSIONS = [
      tfds.core.Version("2.1.0"),
  ]
  RELEASE_NOTES = {
      "2.0.0": "New split API (https://tensorflow.org/datasets/splits)",
  }

  def _info(self):
    features_dict = {
        "image": tfds.features.Image(shape=(64, 64, 1)),
        "label_shape": tfds.features.ClassLabel(num_classes=3),
        "label_scale": tfds.features.ClassLabel(num_classes=6),
        "label_orientation": tfds.features.ClassLabel(num_classes=40),
        "label_x_position": tfds.features.ClassLabel(num_classes=32),
        "label_y_position": tfds.features.ClassLabel(num_classes=32),
        "value_shape": tfds.features.Tensor(shape=[], dtype=np.float32),
        "value_scale": tfds.features.Tensor(shape=[], dtype=np.float32),
        "value_orientation": tfds.features.Tensor(shape=[], dtype=np.float32),
        "value_x_position": tfds.features.Tensor(shape=[], dtype=np.float32),
        "value_y_position": tfds.features.Tensor(shape=[], dtype=np.float32),
    }
    if self.version > "2.0.0":
      features_dict["id"] = tfds.features.Text()
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict(features_dict),
        homepage="https://github.com/deepmind/dsprites-dataset",
    )

  def _split_generators(self, dl_manager):
    filepath = dl_manager.download(_URL)

    # There is no predefined train/val/test split for this dataset.
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN, gen_kwargs=dict(filepath=filepath)
        ),
    ]

  def _generate_examples(self, filepath):
    """Generates examples for the dSprites data set.

    Args:
      filepath: path to the dSprites hdf5 file.

    Yields:
      Dictionaries with images, latent classes, and latent values.
    """
    # Simultaneously iterating through the different data sets in the hdf5
    # file is >100x slower and the data set is small (26.7MB). Hence, we first
    # load everything into memory before yielding the samples.
    with tfds.core.lazy_imports.h5py.File(filepath, "r") as h5dataset:
      image_array = np.array(h5dataset["imgs"])
      class_array = np.array(h5dataset["latents"]["classes"])
      values_array = np.array(h5dataset["latents"]["values"])

    for i, (image, classes, values) in enumerate(
        zip(image_array, class_array, values_array)
    ):
      record = dict(
          image=np.expand_dims(image, -1),
          label_shape=classes[1],
          label_scale=classes[2],
          label_orientation=classes[3],
          label_x_position=classes[4],
          label_y_position=classes[5],
          value_shape=values[1],
          value_scale=values[2],
          value_orientation=values[3],
          value_x_position=values[4],
          value_y_position=values[5],
      )
      if self.version > "2.0.0":
        record["id"] = "{:06d}".format(i)
      yield i, record



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""symmetric_solids dataset."""

import numpy as np
import tensorflow_datasets.public_api as tfds

_DATA_PATH = (
    'https://storage.googleapis.com/gresearch/implicit-pdf/symsol_dataset.zip'
)
_IMAGE_DIMENSIONS = (224, 224, 3)
_SHAPE_NAMES = [
    'tet',
    'cube',
    'icosa',
    'cone',
    'cyl',
    'tetX',
    'cylO',
    'sphereX',
]


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for symmetric_solids dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(
                shape=_IMAGE_DIMENSIONS, dtype=np.uint8
            ),
            'label_shape': tfds.features.ClassLabel(names=_SHAPE_NAMES),
            'rotation': tfds.features.Tensor(shape=(3, 3), dtype=np.float32),
            'rotations_equivalent': tfds.features.Tensor(
                shape=(None, 3, 3), dtype=np.float32
            ),
        }),
        # These are returned if `as_supervised=True` in `builder.as_dataset`.
        supervised_keys=('image', 'rotation'),
        homepage='https://implicit-pdf.github.io',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    extracted_paths = dl_manager.download_and_extract(_DATA_PATH)
    return {
        'train': self._generate_examples(
            images_path=extracted_paths / 'train/images',
            rotations_path=extracted_paths / 'train/rotations.npz',
        ),
        'test': self._generate_examples(
            images_path=extracted_paths / 'test/images',
            rotations_path=extracted_paths / 'test/rotations.npz',
        ),
    }

  def _generate_examples(self, images_path, rotations_path):
    """Yields examples."""
    with rotations_path.open('rb') as f:
      rotations = dict(np.load(f))
    for key in rotations:
      rotations[key] = rotations[key].astype(np.float32)
    for image_path in images_path.glob('*.png'):
      fname = image_path.name
      shape_name, image_index = fname.split('_')
      image_index = int(image_index.split('.')[0])
      shape_id = _SHAPE_NAMES.index(shape_name)
      yield fname, {
          'image': image_path,
          'label_shape': shape_id,
          'rotation': rotations[shape_name][image_index, 0],
          'rotations_equivalent': rotations[shape_name][image_index],
      }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The scicite dataset."""

import json
import os

from etils import epath
import numpy as np
import tensorflow_datasets.public_api as tfds

_SOURCE_NAMES = [
    "properNoun",
    "andPhrase",
    "acronym",
    "etAlPhrase",
    "explicit",
    "acronymParen",
    "nan",
]


class Builder(tfds.core.GeneratorBasedBuilder):
  """This is a dataset for classifying citation intents in academic papers."""

  VERSION = tfds.core.Version("1.0.0")

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "string": tfds.features.Text(),
            "sectionName": tfds.features.Text(),
            "label": tfds.features.ClassLabel(
                names=["method", "background", "result"]
            ),
            "citingPaperId": tfds.features.Text(),
            "citedPaperId": tfds.features.Text(),
            "excerpt_index": np.int32,
            "isKeyCitation": np.bool_,
            "label2": tfds.features.ClassLabel(
                names=["supportive", "not_supportive", "cant_determine", "none"]
            ),
            "citeEnd": np.int64,
            "citeStart": np.int64,
            "source": tfds.features.ClassLabel(names=_SOURCE_NAMES),
            "label_confidence": np.float32,
            "label2_confidence": np.float32,
            "id": tfds.features.Text(),
        }),
        # If there's a common (input, target) tuple from the features,
        # specify them here. They'll be used if as_supervised=True in
        # builder.as_dataset.
        supervised_keys=("string", "label"),
        # Homepage of the dataset for documentation
        homepage="https://github.com/allenai/scicite",
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    dl_paths = dl_manager.download_and_extract(
        {
            "scicite": "https://s3-us-west-2.amazonaws.com/ai2-s2-research/scicite/scicite.tar.gz",
        }
    )
    path = os.path.join(dl_paths["scicite"], "scicite")
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={"path": os.path.join(path, "train.jsonl")},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={"path": os.path.join(path, "dev.jsonl")},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={"path": os.path.join(path, "test.jsonl")},
        ),
    ]

  def _generate_examples(self, path=None):
    """Yields examples."""
    with epath.Path(path).open() as f:
      unique_ids = {}
      for line in f:
        d = json.loads(line)
        unique_id = str(d["unique_id"])
        if unique_id in unique_ids:
          continue
        unique_ids[unique_id] = True
        yield unique_id, {
            "string": d["string"],
            "label": str(d["label"]),
            "sectionName": str(d["sectionName"]),
            "citingPaperId": str(d["citingPaperId"]),
            "citedPaperId": str(d["citedPaperId"]),
            "excerpt_index": int(d["excerpt_index"]),
            "isKeyCitation": bool(d["isKeyCitation"]),
            "label2": str(d.get("label2", "none")),
            "citeEnd": _safe_int(d["citeEnd"]),
            "citeStart": _safe_int(d["citeStart"]),
            "source": str(d["source"]),
            "label_confidence": float(d.get("label_confidence", 0.0)),
            "label2_confidence": float(d.get("label2_confidence", 0.0)),
            "id": str(d["id"]),
        }


def _safe_int(a):
  try:
    # skip NaNs
    return int(a)
  except ValueError:
    return -1



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SUN (Scene UNderstanding) datasets."""

import io
import os

from absl import logging
import numpy as np
from tensorflow_datasets.core import utils
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_SUN397_URL = "https://vision.princeton.edu/projects/2010/SUN/"

# These images are badly encoded and cannot be decoded correctly (TF), or the
# decoding is not deterministic (PIL).
_SUN397_IGNORE_IMAGES = [
    "SUN397/c/church/outdoor/sun_bhenjvsvrtumjuri.jpg",
    "SUN397/t/track/outdoor/sun_aophkoiosslinihb.jpg",
]

_SUN397_BUILDER_CONFIG_DESCRIPTION_PATTERN = (
    "Train and test splits from the official partition number %d. "
    "Images are resized to have at most %s pixels, and compressed with 72 JPEG "
    "quality."
)


def _decode_image(fobj, session, filename):
  """Reads and decodes an image from a file object as a Numpy array.

  The SUN dataset contains images in several formats (despite the fact that
  all of them have .jpg extension). Some of them are:
    - BMP (RGB)
    - PNG (grayscale, RGBA, RGB interlaced)
    - JPEG (RGB)
    - GIF (1-frame RGB)
  Since TFDS assumes that all images have the same number of channels, we
  convert all of them to RGB.

  Args:
    fobj: File object to read from.
    session: TF session used to decode the images.
    filename: Filename of the original image in the archive.

  Returns:
    Numpy array with shape (height, width, channels).
  """

  buf = fobj.read()
  image = tfds.core.lazy_imports.cv2.imdecode(
      np.frombuffer(buf, dtype=np.uint8), flags=3
  )  # Note: Converts to RGB.
  if image is None:
    logging.warning(
        "Image %s could not be decoded by OpenCV, falling back to TF", filename
    )
    try:
      image = tf.image.decode_image(buf, channels=3)
      image = session.run(image)
    except tf.errors.InvalidArgumentError as e:
      raise ValueError(
          f"{e}. Image {filename} could not be decoded by Tensorflow."
      ) from e

  # The GIF images contain a single frame.
  if len(image.shape) == 4:  # rank=4 -> rank=3
    image = image.reshape(image.shape[1:])

  return image


def _encode_jpeg(image, quality=None):
  cv2 = tfds.core.lazy_imports.cv2
  extra_args = [[int(cv2.IMWRITE_JPEG_QUALITY), quality]] if quality else []
  _, buff = cv2.imencode(".jpg", image, *extra_args)
  return io.BytesIO(buff.tobytes())


def _process_image_file(
    fobj, session, filename, quality=None, target_pixels=None
):
  """Process image files from the dataset."""
  # We need to read the image files and convert them to JPEG, since some files
  # actually contain GIF, PNG or BMP data (despite having a .jpg extension) and
  # some encoding options that will make TF crash in general.
  image = _decode_image(fobj, session, filename=filename)
  # Get image height and width.
  height, width, _ = image.shape
  actual_pixels = height * width
  if target_pixels and actual_pixels > target_pixels:
    factor = np.sqrt(target_pixels / actual_pixels)
    image = tfds.core.lazy_imports.cv2.resize(
        image, dsize=None, fx=factor, fy=factor
    )
  return _encode_jpeg(image, quality=quality)


class Sun397Config(tfds.core.BuilderConfig):
  """BuilderConfig for Sun 397 dataset."""

  def __init__(
      self, target_pixels=None, partition=None, quality=None, **kwargs
  ):
    self._target_pixels = target_pixels
    self._partition = partition
    self._quality = quality
    super(Sun397Config, self).__init__(**kwargs)

  @property
  def target_pixels(self):
    return self._target_pixels

  @property
  def partition(self):
    return self._partition

  @property
  def quality(self):
    return self._quality


def _generate_builder_configs():
  """Return the BuilderConfig objects for the SUN397 dataset."""
  version = tfds.core.Version("4.0.0")
  builder_configs = [
      # Images randomly split into train/valid/test splits (70%/10%/20%), and
      # images resized to have at most 120,000 pixels.
      Sun397Config(
          name="tfds",
          version=version,
          target_pixels=120000,
          description=(
              "TFDS partition with random train/validation/test splits with "
              "70%/10%/20% of the images, respectively. Images are resized to "
              "have at most 120,000 pixels, and are compressed with 72 JPEG "
              "quality."
          ),
      ),
  ]
  # Configs for each of the standard partitions of the dataset.
  for partition in range(1, 10 + 1):
    description = _SUN397_BUILDER_CONFIG_DESCRIPTION_PATTERN % (
        partition,
        "120,000",
    )
    builder_configs.append(
        Sun397Config(
            name="standard-part%d-120k" % partition,
            partition=partition,
            target_pixels=120000,
            version=version,
            description=description,
        )
    )
  return builder_configs


class Builder(tfds.core.GeneratorBasedBuilder):
  """Sun397 Scene Recognition Benchmark."""

  BUILDER_CONFIGS = _generate_builder_configs()

  def __init__(self, tfds_split_files=None, **kwargs):
    super(Builder, self).__init__(**kwargs)
    # Note: Only used for tests, since the data is fake.
    if not tfds_split_files:
      tfds_split_files = {
          "tr": "sun397_tfds_tr.txt",
          "te": "sun397_tfds_te.txt",
          "va": "sun397_tfds_va.txt",
      }
      for split, filename in tfds_split_files.items():
        tfds_split_files[split] = tfds.core.tfds_path(
            os.path.join("image_classification", filename)
        )
    self._tfds_split_files = tfds_split_files

  def _info(self):
    names_file = tfds.core.tfds_path(
        os.path.join("image_classification", "sun397_labels.txt")
    )
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "file_name": tfds.features.Text(),
            "image": tfds.features.Image(shape=(None, None, 3)),
            "label": tfds.features.ClassLabel(names_file=names_file),
        }),
        homepage=_SUN397_URL,
    )

  def _split_generators(self, dl_manager):
    paths = dl_manager.download_and_extract({
        "images": tfds.download.Resource(
            url=_SUN397_URL + "SUN397.tar.gz",
            extract_method=tfds.download.ExtractMethod.NO_EXTRACT,
        ),
        "partitions": _SUN397_URL + "download/Partitions.zip",
    })
    if not isinstance(paths, dict):
      # While testing download_and_extract() returns the dir containing the
      # test files.
      paths = {
          "images": os.path.join(paths, "SUN397.tar.gz"),
          "partitions": os.path.join(paths, "Partitions"),
      }
    images = tfds.download.Resource(
        path=paths["images"],
        extract_method=tfds.download.ExtractMethod.TAR_GZ_STREAM,
    )
    if self.builder_config.name == "tfds":
      subset_images = self._get_tfds_subsets_images()
      return [
          tfds.core.SplitGenerator(
              name=tfds.Split.TRAIN,
              gen_kwargs=dict(
                  archive=dl_manager.iter_archive(images),
                  subset_images=subset_images["tr"],
              ),
          ),
          tfds.core.SplitGenerator(
              name=tfds.Split.TEST,
              gen_kwargs=dict(
                  archive=dl_manager.iter_archive(images),
                  subset_images=subset_images["te"],
              ),
          ),
          tfds.core.SplitGenerator(
              name=tfds.Split.VALIDATION,
              gen_kwargs=dict(
                  archive=dl_manager.iter_archive(images),
                  subset_images=subset_images["va"],
              ),
          ),
      ]
    else:
      subset_images = self._get_partition_subsets_images(paths["partitions"])
      return [
          tfds.core.SplitGenerator(
              name=tfds.Split.TRAIN,
              gen_kwargs=dict(
                  archive=dl_manager.iter_archive(images),
                  subset_images=subset_images["tr"],
              ),
          ),
          tfds.core.SplitGenerator(
              name=tfds.Split.TEST,
              gen_kwargs=dict(
                  archive=dl_manager.iter_archive(images),
                  subset_images=subset_images["te"],
              ),
          ),
      ]

  def _generate_examples(self, archive, subset_images):
    prefix_len = len("SUN397")
    with tf.Graph().as_default():
      with utils.nogpu_session() as sess:
        for filepath, fobj in archive:
          if filepath in _SUN397_IGNORE_IMAGES:
            continue
          # Note: all files in the tar.gz are in SUN397/...
          filename = filepath[prefix_len:].replace("\\", "/")  # For windows
          if filename in subset_images:
            # Example:
            # From filename: /c/car_interior/backseat/sun_aenygxwhhmjtisnf.jpg
            # To class: /c/car_interior/backseat
            label = "/".join(filename.split("/")[:-1])
            image = _process_image_file(
                fobj,
                sess,
                filename,
                quality=self.builder_config.quality,
                target_pixels=self.builder_config.target_pixels,
            )
            record = {
                "file_name": filename,
                "image": image,
                "label": label,
            }
            yield filename, record

  def _get_tfds_subsets_images(self):
    splits_sets = {}
    for split, filepath in self._tfds_split_files.items():
      splits_sets[split] = self._load_image_set_from_file(filepath)
    return splits_sets

  def _get_partition_subsets_images(self, partitions_dir):
    # Get the ID of all images in the dataset.
    all_images = set()
    for split_images in self._get_tfds_subsets_images().values():
      all_images.update(split_images)
    # Load the images in the training/test split of this partition.
    filenames = {
        "tr": "Training_%02d.txt" % self.builder_config.partition,
        "te": "Testing_%02d.txt" % self.builder_config.partition,
    }
    splits_sets = {}
    for split, filename in filenames.items():
      filepath = os.path.join(partitions_dir, filename)
      splits_sets[split] = self._load_image_set_from_file(filepath)
    # Put the remaining images in the dataset into the "validation" split.
    splits_sets["va"] = all_images - (splits_sets["tr"] | splits_sets["te"])
    return splits_sets

  def _load_image_set_from_file(self, filepath):
    with tf.io.gfile.GFile(os.fspath(filepath), mode="r") as f:
      return set([line.strip() for line in f])



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""xtreme_pos dataset."""
import os

from tensorflow_datasets.core.dataset_builders.conll import conllu_dataset_builder
from tensorflow_datasets.core.dataset_builders.conll import conllu_dataset_builder_utils as conllu_lib
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_LANGS = {
    "af": "Afrikaans",
    "ar": "Arabic",
    "bg": "Bulgarian",
    "de": "German",
    "el": "Greek",
    "en": "English",
    "es": "Spanish",
    "et": "Estonian",
    "eu": "Basque",
    "fa": "Persian",
    "fi": "Finnish",
    "fr": "French",
    "he": "Hebrew",
    "hi": "Hindi",
    "hu": "Hungarian",
    "id": "Indonesian",
    "it": "Italian",
    "ja": "Japanese",
    "kk": "Kazakh",
    "ko": "Korean",
    "mr": "Marathi",
    "nl": "Dutch",
    "pt": "Portuguese",
    "ru": "Russian",
    "ta": "Tagalog",
    "te": "Telugu",
    "th": "Thai",
    "tl": "Tamil",
    "tr": "Turkish",
    "ur": "Urdu",
    "vi": "Vietnamese",
    "yo": "Yoruba",
    "zh": "Chinese",
}

_DATA_URLS = "https://lindat.mff.cuni.cz/repository/xmlui/bitstream/handle/11234/1-3105/ud-treebanks-v2.5.tgz"


class Builder(tfds.dataset_builders.ConllUDatasetBuilder):
  """DatasetBuilder for xtreme_pos dataset."""

  BUILDER_CONFIGS = []
  for language in _LANGS:
    BUILDER_CONFIGS.append(
        conllu_lib.get_universal_morphology_config(
            language=language,
            features=conllu_lib.XTREME_POS_FEATURES,
            name=f"xtreme_pos_{language}",
        )
    )

  VERSION = tfds.core.Version("1.0.0")
  RELEASE_NOTES = {
      "1.0.0": "Initial release.",
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.create_dataset_info(
        homepage="https://universaldependencies.org/"
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    dl_dir = dl_manager.download_and_extract(_DATA_URLS)
    subpath = os.path.join(dl_dir, "ud-treebanks-v2.5")
    lang = _LANGS[self.builder_config.language]
    data_dir = os.path.join(subpath, "*_" + lang + "*")
    folders = sorted(tf.io.gfile.glob(data_dir))

    if lang == "Kazakh":
      paths = {"train": [], "test": []}
      for split in paths:
        for folder in folders:
          for file in sorted(tf.io.gfile.listdir(folder)):
            if split in file and file.endswith(".conllu"):
              paths[split].append(os.path.join(folder, file))

    elif lang in ["Tagalog", "Thai", "Yoruba"]:
      paths = {"test": []}
      for folder in folders:
        for file in sorted(tf.io.gfile.listdir(folder)):
          if "test" in file and file.endswith(".conllu"):
            paths["test"].append(os.path.join(folder, file))

    else:
      paths = {"train": [], "dev": [], "test": []}
      for split in paths:
        # We exclude Arabic-NYUAD which does not contains any words, only `_`.
        for folder in folders:
          for file in sorted(tf.io.gfile.listdir(folder)):
            if (
                split in file
                and file.endswith(".conllu")
                and "NYUAD" not in folder
            ):
              paths[split].append(os.path.join(folder, file))

    return {  # pylint:disable=g-complex-comprehension
        split: self._generate_examples(
            filepaths=split_files,
            process_example_fn=conllu_dataset_builder.get_xtreme_pos_example,
        )
        for split, split_files in paths.items()
    }
